//========================================================================================
//
//	File:				DlogFrm.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWDIALOG_H
#define FWDIALOG_H

// ----- Framework Includes -----

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWWINDOW_K
#include "FWWindow.k"
#endif

// ----- Foundation Layer -----

#ifndef FWRECEVR_H
#include "FWRecevr.h"
#endif

//==============================================================================
// Forward Declarations
//==============================================================================

class FW_CNotification;
class FW_CIdler;
class FW_CView;
class FW_CViewTabber;
class FW_CButton;
class FW_CVirtualKeyEvent;

//==============================================================================
// class FW_CDialogFrame 
//==============================================================================

class FW_CDialogFrame : public FW_CFrame, public FW_MReceiver
{

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
  public:
	FW_DECLARE_CLASS
	FW_DECLARE_AUTO(FW_CDialogFrame)

	FW_CDialogFrame(Environment* ev, 
					ODFrame* odFrame, 
					FW_CPresentation* presentation, 
					FW_CPart* part,
					FW_ResourceId id = 0);

	virtual ~FW_CDialogFrame();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
	virtual void 		HandleNotification(Environment* ev, const FW_CNotification& notification);

	virtual void 		FrameRemoved(Environment* ev, FW_Boolean toStorage);
	virtual FW_Boolean	DoVirtualKey (Environment* ev, const FW_CVirtualKeyEvent & event);

//----------------------------------------------------------------------------------------
//	New API
//
  public:
  	// ---- Dialog creation
	static FW_CDialogFrame* NewModalDialog(Environment* ev, 
									FW_CPart* part, 
									FW_CPresentation* presentation,
									const FW_CPoint& size, 
									const FW_CPoint& position, 
									FW_WindowStyle style, 
									const FW_CString& windowTitle);
 	 
	static FW_CDialogFrame* NewAndShowModalDialog(Environment* ev, 
									FW_CPart* part, 
									FW_CPresentation* presentation,
									const FW_CPoint& size, 
									const FW_CPoint& position, 
									FW_WindowStyle style, 
									const FW_CString& windowTitle);
  public:
 	// ---- OK/Cancel buttons
	FW_CButton*			GetDefaultButton(Environment* ev) const;
	FW_CButton*			GetCancelButton(Environment* ev) const;
	void				SetDefaultButton(Environment* ev, ODID id);
	void				SetCancelButton(Environment* ev, ODID id);
	
  private:
	FW_Boolean	 		TryModalFocus(Environment* ev, ODFrame* odFrame, FW_Boolean isMoveable);

//----------------------------------------------------------------------------------------
//	Private Implementation
//
public:
	virtual void		PrivButtonAttached(Environment* ev, FW_CButton* button);

	FW_CButton*			PrivSetDismissButton(Environment* ev, 
							FW_CButton* newButton, 
							FW_Message message, 
							FW_CButton* prevButton);

//----------------------------------------------------------------------------------------
//	Data Members
//
  private:
	FW_Boolean			fIsModal;
	FW_CIdler*			fIdler;
	FW_CViewTabber*		fViewTabber;
	FW_CButton*			fDefaultButton;
	FW_CButton*			fCancelButton;
};

//==============================================================================
// FW_CDialogFrame Inlines
//==============================================================================

inline FW_CButton* FW_CDialogFrame::GetDefaultButton(Environment*) const
{
	return fDefaultButton;
}

inline FW_CButton* FW_CDialogFrame::GetCancelButton(Environment*) const
{
	return fCancelButton;
}


#endif
