//========================================================================================
//
//	File:				FWLnkSrc.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWLNKSRC_H
#define FWLNKSRC_H

#ifndef FWLINK_H
#include "FWLink.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class ODStorageUnit;
class ODLinkSource;
class FW_CPresentation;
class FW_CPart;
class FW_CCloneInfo;
class FW_CDataInterchange;
class FW_CPromise;


//========================================================================================
//	class FW_CLinkSource
//========================================================================================

class FW_CLinkSource : public FW_CLink
{
  public:
	FW_CLinkSource(Environment* ev, 
				   ODUpdateID updateID, 
				   FW_CPresentation* presentation);
	virtual ~ FW_CLinkSource();
	
	virtual void		LinkEstablished(Environment* ev) = 0;	
	virtual void		ExternalizeLinkContent(Environment* ev, ODStorageUnit* linkSU) = 0;

	virtual void		ContentUpdated(Environment* ev, 
									   ODUpdateID updateID,
									   FW_Boolean forceUpdate = FALSE);

	virtual FW_Boolean	HasEmbeddedFrame(Environment* ev, ODFrame* odEmbeddedFrame);
	virtual FW_Boolean	IsOKtoUpdate(Environment* ev, FW_Boolean forceUpdate);
 
	virtual void		BreakLink(Environment* ev);
	virtual void		RestoreLink(Environment* ev, FW_CPart* part);

	virtual void		DoExternalizeLink(Environment* ev, ODStorageUnit* storageUnit,
										  FW_CCloneInfo* cloneInfo) = 0;

	void				ExternalizeLink(Environment* ev, ODStorageUnit* storageUnit,
										FW_CCloneInfo* cloneInfo);

	ODLinkSource*		GetODLinkSource(Environment* ev) const;
	void				SetODLinkSource(Environment* ev, ODLinkSource* odLinkSource);

	FW_Boolean			IsPending(Environment* ev) const;
	ODUpdateID			GetUpdateID(Environment* ev) const;
	ODUpdateID			GetPendingID(Environment* ev) const;
	void				SetPendingID(Environment* ev, ODUpdateID id);
	
	FW_Boolean			IsPromised(Environment* ev) const;
	void				ResolveAllPromises(Environment* ev);
	
  public:
	void				PrivLinkEstablished(Environment* ev);

	void				PrivSetPromise(Environment* ev, FW_CPromise* promise);

  protected:
	ODLinkSource*			fODLinkSource;
	ODUpdateID				fUpdateID;
	FW_CDataInterchange*	fDataInterchange;
	
  private:
	ODUpdateID				fPendingID;			// changeID written to link spec
	FW_CPromise*			fPromise;
};

//========================================================================================
//	Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CLinkSource::GetODLinkSource
//----------------------------------------------------------------------------------------
inline ODLinkSource* FW_CLinkSource::GetODLinkSource(Environment*) const
{
	return fODLinkSource;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkSource::IsPending
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CLinkSource::IsPending(Environment*) const
{
	return fODLinkSource == NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkSource::GetUpdateID
//----------------------------------------------------------------------------------------
inline ODUpdateID FW_CLinkSource::GetUpdateID(Environment*) const
{
	return fUpdateID;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkSource::GetPendingID
//----------------------------------------------------------------------------------------
inline ODUpdateID FW_CLinkSource::GetPendingID(Environment*) const
{
	return fPendingID;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkSource::SetPendingID
//----------------------------------------------------------------------------------------
inline void FW_CLinkSource::SetPendingID(Environment*, ODUpdateID id)
{
	fPendingID = id;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkSource::IsPromised
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CLinkSource::IsPromised(Environment*) const
{
	return fPromise != NULL;
}

#endif
