//========================================================================================
//
//	File:				FWProxy.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPROXY_H
#define FWPROXY_H

// ----- ODF OS Includes -----

#ifndef FWFXMATH_H
#include "FWFxMath.h"
#endif

// ----- ODF Foundation Includes -----

#ifndef FWSTDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWCLAINF_H
#include "FWClaInf.h"
#endif

#ifndef FWPRIDEB_H
#include "FWPriDeb.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class ODFrame;
class ODShape;
class ODFacet;
class ODStorageUnitView;
class ODTransform;

class FW_CCloneInfo;
class FW_CFrame;
class FW_CEmbeddingFrame;
class FW_CEmbeddingPart;
class FW_CProxyFrame;
class FW_CPoint;
class FW_CPresentation;
class FW_CString;

//========================================================================================
//	class FW_MProxy
//========================================================================================

class FW_MProxy
{
 public:
	friend class FW_CProxyProxyFrameIterator;
	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_MProxy(Environment* ev, 
				FW_CEmbeddingPart* embeddingPart, 
				FW_CPresentation* presentation,
				FW_Boolean shown = TRUE);
	virtual ~FW_MProxy();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Embedded frames manipulation -----
	void						DetachEmbeddedFrames(Environment *ev);
	void						AttachEmbeddedFrames(Environment *ev);
	void						RemoveEmbeddedFrames(Environment *ev);

	// ----- Notification -----
	virtual void				UsedShapeChanged(Environment *ev,
												FW_CEmbeddingFrame* embeddingFrame,
												ODFrame* odEmbeddedFrame);

	virtual ODShape* 			FrameShapeRequested(Environment* ev, 
												FW_CEmbeddingFrame* frame,
												ODFrame* odEmbeddedFrame, 
												ODShape* requestedFrameShape);
										
	// ----- Query -----
	ODFacet*					GetEmbeddedFacetUnderMouse(Environment* ev, 
									ODFacet* embeddingFacet, 
									const FW_CPoint& mouse) const;	
									
	ODFrame*					AcquireEmbeddedFrame(Environment* ev, 
									FW_CEmbeddingFrame* embeddingFrame) const;
									
	FW_CProxyFrame*				GetProxyFrame(Environment* ev, 
									FW_CEmbeddingFrame* embeddingFrame) const;
	
	FW_Boolean					IsEmbeddedFrame(Environment* ev,
									ODFrame* odEmbeddedFrame) const;
									
	// ----- Active border -----
	virtual void				AdjustBorderShape(Environment* ev,
									FW_CEmbeddingFrame* embeddingFrame, 
									ODFacet* embeddedFacet, 
									ODShape *shape);
									
	// ----- Persistence -----
	void						Externalize(Environment* ev, 
									ODStorageUnitView* destinationSUView,
									FW_CCloneInfo *cloneInfo);
	void						Internalize(Environment* ev, 
									ODStorageUnitView* sourceSUView,
									FW_CCloneInfo *cloneInfo);
																		  
	// ----- Selection state -----
	virtual void				SetSelectState(Environment* ev,
									FW_Boolean state);
	FW_Boolean					GetSelectState(Environment* ev) const;
									
	// ----- Highlight state -----
	void						ChangeHighlight(Environment* ev,
									ODHighlight highlight,
									FW_CEmbeddingFrame* embeddingFrame = NULL);
									
	// ----- Hide/Show Proxy -----
	void						HideShow(Environment* ev, FW_Boolean state);
	FW_Boolean					IsShown(Environment* ev) const;
	
	// ----- Open In Window (From OpenSelection Command) -----
	virtual void				OpenInWindow(Environment* ev, FW_CProxyFrame* proxyFrame);
		
	// ----- Position all embedded frames of this proxy -----
	void 						ChangeExternalTransforms(Environment* ev, 
									ODTransform* transform);
	void 						ChangeExternalTransforms(Environment* ev, 
									FW_Fixed x,
									FW_Fixed y);

	// ----- Change the frame shape of all embedded frames of this proxy -----
	void 						ChangeFrameShapes(Environment* ev, 
									ODShape* newShape);
	void 						ChangeFrameShapes(Environment* ev, 
									FW_Fixed width,
									FW_Fixed height);

	// ----- Set the View Type of this proxy frames -----
	void						SetEmbeddedFramesViewType(Environment *ev, 
									ODTypeToken viewType);

	// ----- Frame Group/Sequence Number -----
	ODID						GetFrameGroup(Environment *ev) const;
	ODID						GetSequenceNumber(Environment *ev) const;
	void						ChangeSequenceNumber(Environment *ev, ODID sequenceNumber);
	
	// ----- Getters/Setters -----
	ODPart*						AcquireEmbeddedPart(Environment* ev) const;
	void						GetEmbeddedPartName(Environment* ev, FW_CString& string) const;
	
	FW_CEmbeddingPart*			GetPart(Environment* ev) const;
	
	// ----- Presentation -----
	FW_CPresentation*			GetPresentation(Environment* ev) const;
	
								
	// ----- Change the link status of all embedded frames of this proxy -----
	void						ChangeLinkStatus(Environment* ev, ODLinkStatus linkStatus);

	// ----- ReleaseAll -----
	virtual void				ReleaseAll(Environment *ev);
	
//----------------------------------------------------------------------------------------
//	Implementation API
//
public:
	// ----- ProxyFrame Factories -----
	FW_CProxyFrame*				PrivNewProxyFrame(Environment* ev, 
												FW_CEmbeddingPart* part,
												ODStorageUnitView* suView);
	FW_CProxyFrame*				PrivNewProxyFrame(Environment* ev, 
												FW_CEmbeddingPart* part, 
												FW_CEmbeddingFrame* containingFrame, 
												ODFrame* embeddedFrame);
	
	void						PrivAddProxyFrame(Environment* ev, FW_CProxyFrame* proxyFrame);
	void						PrivDeleteProxyFrame(Environment* ev, FW_CProxyFrame* proxyFrame);

	FW_CProxyFrame*				PrivGetProxyFrame(Environment* ev, ODStorageUnitID frameID) const;
	FW_CProxyFrame*				PrivContainerIDToProxyFrame(Environment* ev, ODStorageUnitID containerID) const;
	
	void						PrivPostClone(Environment* ev,
									FW_CCloneInfo* cloneInfo);
	
	void						PrivEmbeddedFacetAdded(Environment* ev);
	void						PrivEmbeddedFacetRemoved(Environment* ev);

	void						PrivSetFrameGroup(Environment *ev, ODID frameGroup);

	void						PrivSetDragging(Environment* ev, FW_Boolean state);
															
//----------------------------------------------------------------------------------------
//	Data Members
//----------------------------------------------------------------------------------------

private:
	FW_Boolean								fDetached;
	FW_TOrderedCollection<FW_CProxyFrame>*	fProxyFrames;
	FW_CEmbeddingPart*						fPart;
	FW_CPresentation*						fPresentation;
	FW_Boolean								fSelected;
	ODStorageUnitID							fClonedFrameID;		// Used during cloning
	unsigned short							fEmbeddedFacetCount;
	FW_Boolean								fShown;
	ODID									fFrameGroup;
	ODID									fSequenceNumber;
	FW_Boolean								fDragging;			
};


//========================================================================================
//	FW_MProxy Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetPart
//----------------------------------------------------------------------------------------
inline FW_CEmbeddingPart* FW_MProxy::GetPart(Environment*) const
{
	return fPart;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetPresentation
//----------------------------------------------------------------------------------------
inline FW_CPresentation* FW_MProxy::GetPresentation(Environment*) const
{
	return fPresentation;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetSelectState
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_MProxy::GetSelectState(Environment*) const
{
	return fSelected;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::PrivEmbeddedFacetAdded
//----------------------------------------------------------------------------------------
inline void FW_MProxy::PrivEmbeddedFacetAdded(Environment*)
{
	fEmbeddedFacetCount++;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::PrivEmbeddedFacetRemoved
//----------------------------------------------------------------------------------------
inline void FW_MProxy::PrivEmbeddedFacetRemoved(Environment*)
{
	FW_ASSERT(fEmbeddedFacetCount != 0);
	fEmbeddedFacetCount--;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetFrameGroup
//----------------------------------------------------------------------------------------
inline ODID FW_MProxy::GetFrameGroup(Environment*) const
{
	return fFrameGroup;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetSequenceNumber
//----------------------------------------------------------------------------------------
inline ODID FW_MProxy::GetSequenceNumber(Environment*) const
{
	return fSequenceNumber;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::IsShown
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_MProxy::IsShown(Environment*) const
{
	return fShown;
}

#endif

