//========================================================================================
//
//	File:				FWPxyFrm.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPXYFRM_H
#define FWPXYFRM_H

#ifndef FWODMISS_H
#include "FWODMiss.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class ODFrame;
class ODCanvas;
class ODDraft;
class ODStorageUnitView;
class ODTransform;

class FW_CCloneInfo;
class FW_MProxy;
class FW_CEmbeddingFrame;
class FW_CEmbeddingPart;
class FW_CRect;
class FW_CProxyFrame;

//========================================================================================
//	class FW_CProxyFrame
//========================================================================================

class FW_CProxyFrame
{
	friend class FW_MProxy;
	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
protected:
	FW_CProxyFrame(Environment* ev, 
				FW_MProxy* proxy, 
				FW_CEmbeddingPart* part);
	FW_CProxyFrame(Environment* ev, 
				FW_MProxy* proxy, 
				FW_CEmbeddingPart* part, 
				FW_CEmbeddingFrame* containingFrame, 
				ODFrame* embeddedFrame);
	~FW_CProxyFrame();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	FW_CEmbeddingFrame*	GetContainingFrame(Environment* ev) const;
	ODStorageUnitID		GetContainingFrameID(Environment* ev) const;
	
	void				SetContainingFrame(Environment* ev, FW_CEmbeddingFrame* containingFrame);
	
	FW_MProxy*			GetProxy(Environment* ev) const;
	void				SetProxy(Environment* ev, FW_MProxy* proxy);
	
	FW_Boolean			IsFrameInMemory(Environment* ev) const;

	ODFrame*			GetFrame(Environment* ev) const;
	ODStorageUnitID		GetFrameID(Environment* ev) const;

	FW_Boolean			InternalizeProxyFrame(Environment* ev, ODStorageUnitView* suView, FW_CCloneInfo* cloneInfo);
	FW_Boolean			ExternalizeProxyFrame(Environment* ev, ODStorageUnitView* suView, FW_CCloneInfo* cloneInfo);
	
	void				Release(Environment* ev);

	ODSize				Purge(Environment* ev);
				
	FW_Boolean			IsOrphan(Environment* ev) const;
	
	FW_Boolean			IsVisible(Environment* ev, ODFacet* containingFacet, const FW_CRect& bounds) const;
	FW_Boolean			IsAttached(Environment* ev) const;
	
	void				SetPurgeable(Environment* ev, FW_Boolean purgeable);
	FW_Boolean			IsPurgeable(Environment* ev) const;
			
	void				RemoveAllFacets(Environment* ev);
	void				RemoveFacets(Environment* ev, FW_CEmbeddingFrame* frame, ODFacet* embeddingFacet);

	void				CreateAllFacets(Environment* ev);								
	void				CreateFacets(Environment* ev, FW_CEmbeddingFrame* frame, ODFacet* embeddingFacet);									


	void				SetSequenceNumber(Environment* ev);
	
//----------------------------------------------------------------------------------------
//	Internal API
//
public:	
	void				PrivClose(Environment* ev);

	void				PrivAttach(Environment* ev);	
	void				PrivDetach(Environment* ev);	
	
	void				PrivSetFrameGroup(Environment* ev, ODID frameGroup);
	void				PrivChangeSequenceNumber(Environment* ev, ODID sequenceNumber);
	
//----------------------------------------------------------------------------------------
//	Data Members
//
private:	
	FW_CEmbeddingPart*	fPart;
	
	ODStorageUnitID 	fFrameID;	
	ODDraft*			fFrameDraft;
		
	ODStorageUnitID		fContainingFrameID;

	FW_MProxy*			fProxy;
	ODFrame* 			fEmbeddedFrame;
		
	FW_Boolean			fPurgeable;
	FW_Boolean			fAttached;				// Used by FW_MProxy::Attach/DetachEmbeddedFrames
};


//========================================================================================
// Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsPurgeable
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsPurgeable(Environment*) const
{
	return fPurgeable;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsFrameInMemory
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsFrameInMemory(Environment*) const
{
	return fEmbeddedFrame != NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetFrameID
//----------------------------------------------------------------------------------------
inline ODStorageUnitID FW_CProxyFrame::GetFrameID(Environment*) const
{
	return fFrameID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetProxy
//----------------------------------------------------------------------------------------
inline FW_MProxy* FW_CProxyFrame::GetProxy(Environment*) const
{
	return fProxy;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetContainingFrameID
//----------------------------------------------------------------------------------------
inline ODStorageUnitID FW_CProxyFrame::GetContainingFrameID(Environment*) const
{
	return fContainingFrameID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsOrphan
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsOrphan(Environment*) const
{
	return fContainingFrameID == kODNULLID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsAttached
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsAttached(Environment*) const
{
	return fAttached;
}

#endif
