//========================================================================================
//
//	File:				FWViewAs.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWVIEWAS_H
#define FWVIEWAS_H

#ifndef FWMREFCT_H
#include "FWMRefCt.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

//========================================================================================
// Forward class declarations
//========================================================================================

class ODFacet;
class ODShape;
struct Environment;

class FW_CFrame;
class FW_CShape;
class FW_CPart;
class FW_CPresentation;
class FW_CGraphicContext;

//========================================================================================
// CLASS FW_CViewAs
//========================================================================================

class FW_CViewAs : public FW_MRefCount
{	
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAs(Environment *ev);
	virtual ~FW_CViewAs();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	virtual void		Load(Environment* ev, FW_CFrame* frame) = 0;
	virtual void		Unload(Environment* ev);
	
	virtual ODShape*	CalcUsedShape(Environment* ev) const = 0;
	
	virtual void		Draw(Environment* ev, 
							FW_CFrame* frame,
							ODFacet* facet, 
							ODShape* invalidShape) = 0;
	
//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CShape*			fShape;
};

//========================================================================================
// CLASS FW_CViewAsThumbnail
//========================================================================================

class FW_CViewAsThumbnail : public FW_CViewAs
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsThumbnail(Environment *ev);
	virtual ~FW_CViewAsThumbnail();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		Load(Environment* ev, FW_CFrame* frame);
	virtual ODShape*	CalcUsedShape(Environment* ev) const;
								
	virtual void		Draw(Environment* ev, 
							FW_CFrame* frame,
							ODFacet* facet, 
							ODShape* invalidShape);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_CRect			fUsedRect;		
};

//========================================================================================
// CLASS FW_CViewAsIcon
//========================================================================================

class FW_CViewAsIcon : public FW_CViewAs
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsIcon(Environment *ev, short iconSize);
	virtual ~FW_CViewAsIcon();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		Load(Environment* ev, FW_CFrame* frame);
	virtual ODShape*	CalcUsedShape(Environment* ev) const;
	virtual void		Draw(Environment* ev,
							FW_CFrame* frame, 
							ODFacet* facet, 
							ODShape* invalidShape);
	
//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	short				fIconSize;
};


//========================================================================================
// CLASS FW_CViewAsSmallIcon
//========================================================================================

class FW_CViewAsSmallIcon : public FW_CViewAsIcon
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsSmallIcon(Environment *ev);
	virtual ~FW_CViewAsSmallIcon();
};

//========================================================================================
// CLASS FW_CViewAsLargeIcon
//========================================================================================

class FW_CViewAsLargeIcon : public FW_CViewAsIcon
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsLargeIcon(Environment *ev);
	virtual ~FW_CViewAsLargeIcon();	
};

#endif