//========================================================================================
//
//	File:				FWClpCmd.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWCLPCMD_H
#include "FWClpCmd.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWINTER_H
#include "FWInter.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWLINK_H
#include "FWLink.h"
#endif

#ifndef FWLNKMGR_H
#include "FWLnkMgr.h"
#endif

// ----- OS Layer -----

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

#ifndef SLODFSTR_K
#include "SLODFStr.k"
#endif

#ifndef SLODFSTR_H
#include "SLODFStr.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_Commands_defined
#include <CmdDefs.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODClipboard_xh
#include <Clipbd.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#ifndef SOM_ODLinkSpec_xh
#include <LinkSpec.xh>
#endif

//========================================================================================
//	Runtime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfcommands
#endif

FW_DEFINE_AUTO(FW_CClipboardCommand)

//====================================================================
// FW_CClipboardCommand class
//====================================================================

//--------------------------------------------------------------------
// FW_CClipboardCommand constructor
//--------------------------------------------------------------------

FW_CClipboardCommand::FW_CClipboardCommand(Environment* ev,
								ODCommandID id,
								FW_CFrame* frame, 
								FW_Boolean canUndo) 
:	FW_CCommand(ev, id, frame, id == kODCommandCopy ? FALSE :canUndo),
	fUpdateID(kODUnknownUpdate),
	fCloneKind(kODCloneCopy),
	fPasteAsHandler(NULL),
	fClipboard(frame->GetPart(ev)->GetSession(ev)->GetClipboard(ev)),
	fSelection(NULL)
{
	short undoMsgID;
	
	switch (id)
	{
		case kODCommandCopy:
			SetCausesChange(ev, FALSE);		// Copy doesn't change the data
			break;
			
		case kODCommandCut:
			fCloneKind = kODCloneCut;
			undoMsgID = FW_kUndoCutMsg;
			break;
			
		case kODCommandPaste:
			fCloneKind = kODClonePaste;
			undoMsgID = FW_kUndoPasteMsg;
			break;
			
		case kODCommandPasteAs:
			fCloneKind = kODClonePaste;
			undoMsgID = FW_kUndoPasteAsMsg;
			break;
			
		case kODCommandClear:
			undoMsgID = FW_kUndoClearMsg;
			break;
		
		default:
			undoMsgID = FW_kDefaultUndoMsg;
			break;
	}

	fSelection = frame->GetPresentation(ev)->GetSelection(ev);
	
	if (GetCanUndo(ev))
	{
		FW_CString undoString;
		FW_CString redoString;
		
		::FW_PrivLoadUndoStrings(ev, undoMsgID, undoString, redoString);
		SetMenuStrings(ev, undoString, redoString);
	}
	FW_END_CONSTRUCTOR
}

//--------------------------------------------------------------------
// FW_CClipboardCommand destructor
//--------------------------------------------------------------------

FW_CClipboardCommand::~FW_CClipboardCommand()
{
	FW_START_DESTRUCTOR
	delete fPasteAsHandler;
}

//--------------------------------------------------------------------
// FW_CClipboardCommand::DoIt
//--------------------------------------------------------------------

void FW_CClipboardCommand::DoIt(Environment* ev)	// Override
{
	FW_Boolean result = FALSE;

	if (GetCanUndo(ev))
		this->SaveUndoState(ev);	// save current state, for later Undo

	switch (GetCommandID(ev))
	{
		case kODCommandCopy:
			this->Copy(ev);
			break;

		case kODCommandClear:
			if (this->IsOKtoEdit(ev))
			{
				this->Clear(ev);
				result = TRUE;
				GetPresentation(ev)->ContentUpdated(ev);
			}
			break;

		case kODCommandCut:
			if (this->IsOKtoEdit(ev))
			{
				this->Cut(ev);
				result = TRUE;
				GetPresentation(ev)->ContentUpdated(ev);
			}
			break;

		case kODCommandPaste:
			if (this->IsOKtoEdit(ev) && this->Paste(ev))
			{
				result = TRUE;
				GetPresentation(ev)->ContentUpdated(ev);
			}
			break;

		case kODCommandPasteAs:
			if (this->IsOKtoEdit(ev) && this->PasteAs(ev))
			{
				result = TRUE;
				GetPresentation(ev)->ContentUpdated(ev);
			}
			break;
	}

	if (result == FALSE)
		SetCanUndo(ev, FALSE);
	else if (GetCanUndo(ev))
	{
		this->SaveRedoState(ev);		// save new state, for later Redo
	}
}


//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::Cut
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::Cut(Environment* ev)
{
	PreCommand(ev);
	
	this->PrivCopy(ev, false);	// don't write a link spec for a cut
	fSelection->ClearSelection(ev);
	
	// Give the command subclass a chance to do something
	this->CommandDone(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::Copy
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::Copy(Environment* ev)
{
	PreCommand(ev);
	
	FW_Boolean allowLinking = (fSelection && fSelection->IsSelectionLinkable(ev));
	this->PrivCopy(ev, allowLinking);
	
	CommandDone(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::Clear
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::Clear(Environment* ev)
{
	PreCommand(ev);
	fSelection->ClearSelection(ev);
	CommandDone(ev);	// Give the command subclass a chance to do something
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::Paste
//----------------------------------------------------------------------------------------

FW_Boolean FW_CClipboardCommand::Paste(Environment* ev)
{
	PreCommand(ev);
	return PrivPaste(ev);	// will call CommandDone
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::PrivPaste
//----------------------------------------------------------------------------------------

FW_Boolean FW_CClipboardCommand::PrivPaste(Environment* ev)
{
	FW_Boolean result = FALSE;
	
	ODStorageUnit* clipBSU = fClipboard->GetContentStorageUnit(ev);

	result = GetPart(ev)->GetDataInterchange(ev)->InternalizeData(ev, fSelection->GetSelectedContent(ev), GetFrame(ev), clipBSU, FW_kClipboardStorage, fCloneKind, NULL) != FW_kInternalizeFailed;

	if (result)	
	{
		// notify OpenDoc clipboard
		fUpdateID = fClipboard->ActionDone(ev, fCloneKind);

		// Give the command subclass a chance to do something
		CommandDone(ev);
	}

	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::PasteAs
//----------------------------------------------------------------------------------------

FW_Boolean FW_CClipboardCommand::PasteAs(Environment* ev)
{
	FW_Boolean result = false;
	FW_VOLATILE(result);
	FW_Boolean handledIt = false;

	PreCommand(ev);

	FW_TRY
	{
		// Create a PasteAs handler object to do some of the work
		fPasteAsHandler = new FW_MPasteAsHandler(ev, GetFrame(ev));
		result = fPasteAsHandler->HandlePasteAsDialog(ev, fCloneKind, handledIt);
		if (!handledIt)
		{
			SetCommandID(ev, kODCommandPaste);
			result = this->PrivPaste(ev);		// just an ordinary paste
		}
		else if (result)
		{
			// notify OpenDoc clipboard
			fUpdateID = fClipboard->ActionDone(ev, fCloneKind);
	
			// if a link was created, need to post an End transaction
			if (this->GetNewLink(ev) != NULL)
			{
				SetActionType(ev, kODEndAction);
			}

			// Give the command subclass a chance to do something
			this->CommandDone(ev);
		}
	}
	FW_CATCH_BEGIN
	FW_CATCH_EVERYTHING()
	{
		result = false;	// don't re-throw, just return false
	}
	FW_CATCH_END

	if (result == false || GetCommandID(ev) == kODCommandPaste)
	{
		// PasteAs handler either not used or not needed
		delete fPasteAsHandler;
		fPasteAsHandler = NULL;
	}
	
	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::PrivCopy
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::PrivCopy(Environment* ev, FW_Boolean allowLinking)
{
	//---- We have to invalidate any currently pending link ----
	FW_CLinkManager* linkMgr = GetPart(ev)->GetLinkManager(ev);
	if (linkMgr)
		linkMgr->DeletePendingClipboardLink(ev);

	//---- Clear any leftover promises on the clipboard ----
	fClipboard->GetContentStorageUnit(ev)->ClearAllPromises(ev);
	GetPart(ev)->GetDataInterchange(ev)->PrivDeletePromises(ev, FW_kClipboardStorage);

	//---- Clear all properties and values from the clipboard ----
	fClipboard->Clear(ev);
	
	// [HLX] Getting the clipboardSU before calling fClipboard->Clear(ev) result in 
	// a crash in ExternalizeData when I try to get the draft?????
	ODStorageUnit* clipboardSU = fClipboard->GetContentStorageUnit(ev);
	
	//---- Write new data to the clipboard ----
	GetPart(ev)->GetDataInterchange(ev)->ExternalizeData(ev, 
													fSelection->GetSelectedContent(ev), 
													GetFrame(ev), 
													clipboardSU, 
													FW_kClipboardStorage, 
													fCloneKind);

	//---- notify OpenDoc clipboard ----
	fUpdateID = fClipboard->ActionDone(ev, fCloneKind);
			
	// ----- Write out Link Spec if possible -----
	if (allowLinking)
	{
		ODLinkSpec* linkSpec = NULL;
		FW_VOLATILE(linkSpec);
		ODUpdateID updateID = fClipboard->GetUpdateID(ev);
		FW_CByteArray data(&updateID, sizeof(updateID));

		FW_TRY
		{
			linkSpec = GetPart(ev)->GetDraft(ev)->CreateLinkSpec(ev, GetPart(ev)->GetODPart(ev), data);

			clipboardSU->AddProperty(ev, kODPropLinkSpec);
			linkSpec->WriteLinkSpec(ev, clipboardSU);
		}
		FW_CATCH_BEGIN
		FW_CATCH_EVERYTHING()
		{
			if (linkSpec)
				delete linkSpec;
			FW_THROW_SAME();
		}
		FW_CATCH_END

		delete linkSpec;

		// If current selection is already published, just re-use its link
		FW_CLinkSource* linkSource = fSelection->DoFindLinkSource(ev);
		if (linkSource)
			linkSource->SetPendingID(ev, updateID);
		else
			linkSource = linkMgr->NewLinkSource(ev, updateID, GetPresentation(ev));
		linkMgr->SetPendingClipboardLink(ev, linkSource);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::UndoIt
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::UndoIt(Environment* ev)	// Override
{
	switch (GetCommandID(ev))
	{
		case kODCommandCut:
		case kODCommandPaste:
		case kODCommandPasteAs:
		{
			fClipboard->ActionUndone(ev, fUpdateID, fCloneKind);
		}
	}
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::RedoIt
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::RedoIt(Environment* ev)	// Override
{
	switch (GetCommandID(ev))
	{
		case kODCommandCut:
		case kODCommandPaste:
		case kODCommandPasteAs:
		{
			fClipboard->ActionRedone(ev, fUpdateID, fCloneKind);
		}
	}
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::PreCommand
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::PreCommand(Environment* ev)
{
FW_UNUSED(ev);
	// User may override; 
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::CommandDone
//----------------------------------------------------------------------------------------

void FW_CClipboardCommand::CommandDone(Environment* ev)
{
FW_UNUSED(ev);
	// User may override; only gets called if the Command was successful
}

//----------------------------------------------------------------------------------------
//	FW_CClipboardCommand::GetNewLink
//----------------------------------------------------------------------------------------

FW_CLinkDestination* FW_CClipboardCommand::GetNewLink(Environment* ev) const
{
	return fPasteAsHandler ? fPasteAsHandler->GetNewLink(ev) : NULL;
}
