//========================================================================================
//
//	File:				FWFrame.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWPRTITE_H
#include "FWPrtIte.h"
#endif

#ifndef FWPRHDLR_H
#include "FWPrHdlr.h"
#endif

#ifndef FWCLPCMD_H
#include "FWClpCmd.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef SLMACOS_H
#include "SLMacOS.h"
#endif

#ifndef FWFLOWIN_H
#include "FWFloWin.h"
#endif

#ifndef FWODGEOM_H
#include "FWODGeom.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

#ifndef FWVIEWAS_H
#include "FWViewAs.h"
#endif

#ifndef FWCURSOR_H
#include "FWCursor.h"
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

#ifndef FWSCROLR_H
#include "FWScrolr.h"
#endif

#ifndef FWCNTRHW_H
#include "FWCntrHW.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWIDLE_H
#include "FWIdle.h"
#endif

#ifndef FWCONTXT_H
#include "FWContxt.h"
#endif

#ifndef FWRESOUR_H
#include "FWResour.h"
#endif

#ifndef FWRGNSHP_H
#include "FWRgnShp.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_Module_OpenDoc_Commands_defined
#include <CmdDefs.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_ODArbitrator_xh
#include <Arbitrat.xh>
#endif

#ifndef SOM_ODStorageUnitView_xh
#include <SUView.xh>
#endif

#ifndef SOM_ODClipboard_xh
#include <Clipbd.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#if defined(__MWERKS__) && GENERATING68K
// A hack to work around a bug
#pragma import list somGetGlobalEnvironment
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment FW_FrameSegment
#endif

//========================================================================================
//	Template Instantiations
//========================================================================================

FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollectionIterator, FW_CFrame)
FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollection, FW_CFrame)

#ifdef FW_USE_TEMPLATE_PRAGMAS

#pragma template_access public
#pragma template FW_TOrderedCollection<FW_CFrame>
#pragma template FW_TOrderedCollectionIterator<FW_CFrame>

#endif

//========================================================================================
//	Constant
//========================================================================================

typedef long FW_FrameVersion;
const FW_FrameVersion FW_kFrameVersion = 0;

//========================================================================================
//	class FW_CFrame
//========================================================================================

FW_DEFINE_AUTO(FW_CFrame)
FW_DEFINE_CLASS_M1(FW_CFrame, FW_CSuperView)

//----------------------------------------------------------------------------------------
//	FW_CFrame::FW_CFrame
//----------------------------------------------------------------------------------------

FW_CFrame::FW_CFrame(Environment *ev,
					ODFrame* odFrame, 
					FW_CPresentation* presentation, 
					FW_CPart* part,
					FW_ResourceId id) :
	FW_CSuperView(ev),		
	fFrameID(odFrame->GetID(ev)),
	fODFrame(odFrame),
	fPart(part),
	fResourceId(id),
	fContentView(NULL),
	fGetFrontClicks(TRUE),
	fNeedsFoci(FALSE),
	fHasSelectionFocus(FALSE),
	fActiveFacet(NULL),
	fCanBeActiveFrame(TRUE),	
	fPresentation(presentation),
	fWindow(NULL),
	fFacetCount(0),
	fPreviousViewType(odFrame->GetViewType(ev)),
	fViewAs(NULL),
	fIdler(NULL),
	fDraggable(NULL),
	fDroppable(NULL),
	fScroller(NULL),	
	fViewUnderCursor(NULL),
	fRequestedFrame(FALSE),	
	fInSharedWindow(FALSE),
#ifdef FW_BUILD_WIN
	fShadowWindow(NULL),
#endif	
	fInternalTransformRealigmentNeeded(FALSE),
	fContentViewOffset(FW_kZeroPoint),
	fSubViewsCreated(FALSE),
	fSourceFrame(this),
	fDroppableState(FW_kFrameDroppable)
{
	FW_ASSERT(fODFrame);
	FW_ASSERT(fPart != NULL);
	
	// Frame is the default target of the event handling chain
	// (it will be the Content View later if one is created)
	fTarget = this;
	
	this->SetNextEventHandler(ev, part);

	fNeedsFoci = odFrame->IsRoot(ev);
	
	SetViewId(ev, fPresentation->GetPresentationType(ev));
	
	// ----- force to be view as frame if root -----
	if (fPreviousViewType != FW_CPart::gViewAsFrameToken && IsRoot(ev))
		fODFrame->SetViewType(ev, FW_CPart::gViewAsFrameToken);

	// ----- Set the frame's view Bounds to its shape bounds
	FW_CRect frameRect;
	PrivGetFrameShapeBounds(ev, frameRect);
	PrivSetBounds(ev, frameRect);
	PrivSetExtent(ev, frameRect.Size());	

	// ----- By default the frame is the ContentView
	PrivSetIsContentView(ev, TRUE);
	fContentView = (FW_CSuperView*)this;
	
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::~FW_CFrame
//----------------------------------------------------------------------------------------

FW_CFrame::~FW_CFrame()
{
	FW_START_DESTRUCTOR

	Environment* ev = somGetGlobalEnvironment();
	
	FW_ASSERT(fIdler == NULL);	// subclass must delete idler it created

	FW_ASSERT(fWindow == NULL);	// should have been deleted in FrameRemoved
	
	if (fViewAs && fPresentation)
		fPresentation->PrivReleaseViewAs(fViewAs);

	// Delete first all subviews before deleting the scroller. 
	DeleteSubViews(ev);
	
	// Now I can delete the scroller (if any)	
	delete fScroller;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::FrameRemoved
//----------------------------------------------------------------------------------------
// Notification

void FW_CFrame::FrameRemoved(Environment* ev, FW_Boolean toStorage)
{
FW_UNUSED(toStorage);

//	if (toStorage && fIdler)	[LSD] why this test?
	if (fIdler)
		fIdler->UnregisterIdle(ev);
		
	// ----- Relinquish what ever focus we have
	RelinquishFocusSet(ev);
		
	// ----- Check that the window has been deleted -----
	FW_ASSERT(fFacetCount == 0);	
	FW_ASSERT(fWindow == NULL);	// When the frame is removed it should not have any facets. fWindow
								// should already be null (see FacetRemoved)
	
	fODFrame = NULL;			// Don't have an ODFFrame anymore
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::FrameAdded
//----------------------------------------------------------------------------------------
// Notification

void FW_CFrame::FrameAdded(Environment* ev, ODFrame* odFrame, FW_Boolean fromStorage)
{
FW_UNUSED(fromStorage);
		
	// In the case of frame in a shared window fODFrame can be NULL
	FW_ASSERT(fODFrame == odFrame || fODFrame == NULL);
	fODFrame = odFrame;
	
	// ----- Add it to the presentation -----
	fPresentation->PrivAddFrame(ev, this);
	
	// ----- Initialize printing handler -----
	FW_CPrintHandler* printHandler = NewPrintHandler(ev);
	if (printHandler != NULL)
		AdoptEventHandler(ev, printHandler);
}

//----------------------------------------------------------------------------------------
// FW_CFrame::GetTarget
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_CFrame::GetTarget(Environment* ev) const
{
FW_UNUSED(ev);

	return fTarget;		// cannot be NULL
}

//----------------------------------------------------------------------------------------
// FW_CFrame::SetTarget
//----------------------------------------------------------------------------------------

void FW_CFrame::SetTarget(Environment* ev, FW_MEventHandler* theTarget)
{
FW_UNUSED(ev);

	FW_ASSERT(theTarget != NULL);
	fTarget = theTarget;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::WantsToBeTarget
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::WantsToBeTarget(Environment* ev)
{
FW_UNUSED(ev);

	return TRUE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleMenu
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HandleMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	FW_Boolean result = TRUE;
	FW_CClipboardCommand* cmd;
	ODCommandID commandID = theMenuEvent.GetCommandID(ev);
	
	switch (commandID)
	{
		case kODCommandViewAsWin:
			fPresentation->OpenPartWindow(ev, this, NULL);
			break;
		
		case kODCommandCut:
		case kODCommandCopy:
		case kODCommandPaste:
		case kODCommandPasteAs:
		case kODCommandClear:
			cmd = this->NewClipboardCommand(ev, commandID);
			if (cmd)
				cmd->Execute(ev);
			break;
						
		case kODCommandSelectAll:
			FW_CSelection* selection = fPresentation->GetSelection(ev);
			if (selection)
				selection->SelectAll(ev);
			break;
			
		default:
			result = FW_CSuperView::HandleMenu(ev, theMenuEvent);	
	}
	
	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::NewPartWindow
//----------------------------------------------------------------------------------------

FW_CWindow* FW_CFrame::NewPartWindow(Environment* ev, ODFacet* sourceFacet)
{
	FW_CRect frameBounds = GetBounds(ev);
	FW_FrameToWindow(ev, sourceFacet, frameBounds);

	GetWindow(ev)->WindowToScreen(ev, frameBounds);
	frameBounds.Offset(FW_IntToFixed(20), FW_IntToFixed(20));
	
	FW_CString255 partName;
	fPart->GetPartName(ev, partName);
	
	return new FW_CWindow(ev, 
						fPart,
						fODFrame,
						FALSE,							// persistent frame
						FW_CPart::gViewAsFrameToken,
						GetPresentation(ev),
						partName,
						frameBounds.Size(),
						frameBounds.TopLeft(),
						FW_kDocumentWindow);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::NewClipboardCommand
//----------------------------------------------------------------------------------------

FW_CClipboardCommand* FW_CFrame::NewClipboardCommand(Environment* ev, ODCommandID commandID)
{
FW_UNUSED(ev);
FW_UNUSED(commandID);
	// [LSD] removed the assert to be more flexible in Form's CEditCmd class
	//		 user should check the return value before calling command->Execute(ev) 
//	FW_DEBUG_MESSAGE("Your need to override FW_CFrame::NewClipboardCommand to create a command");
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::InstallMenus
//----------------------------------------------------------------------------------------
// 	By default call the part with the part menuBar

void FW_CFrame::InstallMenus(Environment *ev)
{
	GetPart(ev)->InstallMenus(ev, GetPart(ev)->GetMenuBar(ev));
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleWindowEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HandleWindowEvent(Environment* ev, const FW_CMacWindowEvent& windowEvent)
{
	FW_Boolean handled = TRUE;
	
	unsigned long message = windowEvent.GetMessage();
	switch (windowEvent.GetMessage())
	{
		case inZoomIn:
		case inZoomOut:
			{	
				ODPlatformWindow platformWindow = GetWindow(ev)->GetPlatformWindow(ev);
				
				// ----- Get the size of the window border -----
				FW_CRect borderSize;
				FW_PlatformError error;
				FW_GetWindowBorderSize(platformWindow, borderSize, &error);
				FW_FailOnError(error);
				
				// ----- Find out where the window would like to be zoomed out to 
				FW_CRect screenRect;
				::FW_MacGetMaxIntersectedDevice(platformWindow, screenRect, &error);
				FW_FailOnError(error);
				screenRect.Inset(FW_IntToFixed(3), FW_IntToFixed(3));	// leave space around window structure area
			
				// ----- call the part to get a custom size (interior size) -----
				FW_CPoint proposedSize(screenRect.Width() - borderSize.left - borderSize.right, 
									   screenRect.Height() - borderSize.top - borderSize.bottom);
				this->AdjustZoomedWindowSize(ev, proposedSize);
				
				fWindow->PrivMacDoZoom(ev, proposedSize, borderSize, screenRect, message);
			}
			break;
		
		case inGrow:
			PrivMacInGrowBox(ev, windowEvent);
			break;

		case inGoAway:
 			if (fWindow->IsFloating(ev)) 
				fWindow->Hide(ev);
			else
				handled = FALSE;
			break;
			
		default:
			handled = FALSE;
	}
	
	return handled;
}
#endif

//----------------------------------------------------------------------------------------
// FW_CFrame::AdjustZoomedWindowSize
//----------------------------------------------------------------------------------------
//	proposedSize is the interior size of the window

void FW_CFrame::AdjustZoomedWindowSize(Environment *ev, FW_CPoint& proposedSize)
{
FW_UNUSED(ev);
FW_UNUSED(proposedSize);
}

//----------------------------------------------------------------------------------------
// FW_CFrame::AdjustWindowGrowLimits
//----------------------------------------------------------------------------------------
//	proposedSize is the interior size of the window

void FW_CFrame::AdjustWindowGrowLimits(Environment* ev, FW_CPoint& minSize, FW_CPoint& maxSize)
{
FW_UNUSED(ev);
FW_UNUSED(minSize);
FW_UNUSED(maxSize);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HandleAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, 
										FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
	if (hasMenuFocus)
	{
		// ----- Only if the target is the content view -----
		if (GetTarget(ev) == fContentView)	
		{
			FW_CSelection* selection = fPresentation->GetSelection(ev);
			FW_Boolean hasSelection = selection && !selection->IsEmpty(ev);
		
			// ----- Edit Menu -----
			// If read only clear, cut past items will be disabled in FW_CPart::HandleAdjustMenus
			menuBar->EnableCommand(ev, kODCommandClear, hasSelection);
			menuBar->EnableCommand(ev, kODCommandCut, hasSelection);
			menuBar->EnableCommand(ev, kODCommandCopy, hasSelection);
			
			menuBar->EnableCommand(ev, kODCommandPasteAs, HasPropertyOnClipboard(ev, kODPropLinkSpec, kODLinkSpec));
		}
		
		menuBar->EnableCommand(ev, kODCommandViewAsWin, !IsTopFrame(ev));
	}
	
	return FW_CSuperView::HandleAdjustMenus(ev, menuBar, hasMenuFocus, isRoot);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsTopFrame
//----------------------------------------------------------------------------------------
//	IsTopFrame returns true if this frame is a sub-frame of the root frame

FW_Boolean FW_CFrame::IsTopFrame(Environment* ev) const
{
	if (IsRoot(ev))
		return TRUE;
	
	FW_CAcquiredODFrame aqContainingFrame = AcquireContainingFrame(ev);
	return IsSubframe(ev) && aqContainingFrame->IsRoot(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsSiblingFrameOf
//----------------------------------------------------------------------------------------
//	Look if both frames are from the same part and have the same presentation. Returns FALSE
//	if otherODFrame is NULL.

FW_Boolean FW_CFrame::IsSiblingFrameOf(Environment *ev, ODFrame* otherODFrame) const
{
	if (otherODFrame == NULL)
		return FALSE;
		
	FW_CAcquiredODPart otherPart = otherODFrame->AcquirePart(ev);
	if (GetODPart(ev) == otherPart)
	{
		FW_CFrame* otherFrame = FW_CFrame::ODtoFWFrame(ev, otherODFrame);
		return GetPresentation(ev) == otherFrame->GetPresentation(ev);
	}
	
	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HasSamePartEditorAs
//----------------------------------------------------------------------------------------
//	Look if the presentation of the otherFrame is the same as mine. Returns TRUE even if they
//	have two different instances of the editor.

FW_Boolean FW_CFrame::HasSamePartEditorAs(Environment *ev, ODFrame* otherFrame) const
{
	if (otherFrame != NULL)
	{
		ODTypeToken proposedToken = otherFrame->GetPresentation(ev);
		FW_CPartPresentationIterator ite(fPart);
		for (FW_CPresentation* presentation = ite.First();
			ite.IsNotComplete();
			presentation = ite.Next())
		{
			if (presentation->GetPresentationType(ev) == proposedToken)
				return TRUE;
		}
	}
	
	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::FocusStateChanged
//----------------------------------------------------------------------------------------
//	When overridden, call inherited::FocusStateChanged FIRST

void FW_CFrame::FocusStateChanged(Environment *ev, ODTypeToken focus, FW_Boolean newState, ODFrame* newOwner)
{
	if (focus == FW_CPart::gSelectionFocusToken)
	{
		fPart->PrivSetLastActiveFrame(this);

		ODWindowState* windowState =  fPart->GetSession(ev)->GetWindowState(ev);
		
		if (newState)
		{
			fHasSelectionFocus = TRUE;
			GetTarget(ev)->ActivateTarget(ev);
			
			// Handle case of multiple parts sharing the same floating windows
			FW_CFloatingWindow::TransferOwnership(ev, windowState, fPart->GetODPart(ev));
			
			// Show the floating windows except if frame has modal focus (i.e. Dialogs)
			if (!HasModalFocus(ev))
				FW_CFloatingWindow::HideShowFloatingWindows(ev, windowState, TRUE);
		}
		else
		{
			// 	Close the selection if I become deactive and the newOwner is not NULL and
			//	not one of my sibling and there is not drag pending
			FW_CSelection* selection = fPresentation->GetSelection(ev);
			if (selection && !this->IsSiblingFrameOf(ev, newOwner) && !fPresentation->PrivIsDragPending())
				selection->CloseSelection(ev);

			fHasSelectionFocus = FALSE;
			GetTarget(ev)->DeactivateTarget(ev);
			
			if (!HasSamePartEditorAs(ev, newOwner))
				FW_CFloatingWindow::HideShowFloatingWindows(ev, windowState, FALSE);
		}
	}
	else if (focus == FW_CPart::gMenuFocusToken)
	{
		if (newState)
			InstallMenus(ev);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::CanBeActiveFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::CanBeActiveFrame(Environment* ev) const
{
	FW_ASSERT(GetWindow(ev) != NULL);
	return fCanBeActiveFrame && !GetWindow(ev)->IsFloating(ev) && !IsFrozen(ev) && (GetViewType(ev) == FW_CPart::gViewAsFrameToken);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ODtoFWFrame
//----------------------------------------------------------------------------------------

FW_CFrame* FW_CFrame::ODtoFWFrame(Environment *ev, ODFrame* odFrame)
{
	return odFrame ? (FW_CFrame*)odFrame->GetPartInfo(ev) : NULL;
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::FrameShapeChanged
//----------------------------------------------------------------------------------------
//	When overridden, call inherited::FrameShapeChanged first

void FW_CFrame::FrameShapeChanged(Environment* ev)
{
	FW_CRect frameRect;
	PrivGetFrameShapeBounds(ev, frameRect);
	FW_CPoint newSize = frameRect.Size();
	
	if (GetViewType(ev) == FW_CPart::gViewAsFrameToken)
	{
#ifdef FW_BUILD_WIN
		MoveSizeShadowWindow(ev);
#endif
		// Change the size and refresh modified region.  (This will in turn call SetExtent 
		// and adjust the layout of subviews if the frame is not the content view)
		SetSize(ev, newSize, TRUE);
	}
	else
	{
		// Use the FW_CView method to avoid resizing the frame's subviews
		// when it's not displayed as a frame
		FW_CView::SetSize(ev, newSize, TRUE);
	}
	
	UpdateUsedAndActiveShapes(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetActiveFacet
//----------------------------------------------------------------------------------------

void FW_CFrame::SetActiveFacet(Environment* ev, ODFacet*  activeFacet)
{
FW_UNUSED(ev);
	fActiveFacet = activeFacet;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::UpdateUsedAndActiveShapes
//----------------------------------------------------------------------------------------

void FW_CFrame::UpdateUsedAndActiveShapes(Environment* ev)
{
	// ----- Used Shape -----
	UpdateUsedShape(ev);

	// ----- Update Active Shape for every facets -----
	FW_CFrameFacetIterator iter(ev, this);
	for (ODFacet* facet = iter.First(ev); iter.IsNotComplete(ev); facet = iter.Next(ev))
	{
		UpdateActiveShape(ev, facet);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::UpdateUsedShape
//----------------------------------------------------------------------------------------

void FW_CFrame::UpdateUsedShape(Environment* ev)
{
	// ----- Used Shape -----
	FW_CAcquiredODShape aqUsedShape;
	
	PrivCreateViewAs(ev);
	if (fViewAs == NULL)
	{
		//	----- By default the used shape is the frame shape
		FW_CAcquiredODShape aqFrameShape = FW_CopyAndRelease(ev, AcquireFrameShape(ev, NULL));
		aqUsedShape = AdjustUsedShape(ev, aqFrameShape);
	}
	else
	{
		aqUsedShape = fViewAs->CalcUsedShape(ev);		// returns a copy			
	}
	
	// ----- if aqUsedShape is NULL OpenDoc will use the frame shape
	ChangeUsedShape(ev, aqUsedShape, NULL);		
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::UpdateActiveShape
//----------------------------------------------------------------------------------------

void FW_CFrame::UpdateActiveShape(Environment* ev, ODFacet* facet)
{
	FW_CAcquiredODShape aqActiveShape;
	
	// ---- the Active shape is a copy of the Used shape by default
	FW_CAcquiredODShape aqProposedActiveShape = FW_CopyAndRelease(ev, AcquireUsedShape(ev));
	
	PrivCreateViewAs(ev);
	if (fViewAs == NULL) 
		aqActiveShape = AdjustActiveShape(ev, facet, aqProposedActiveShape);
	else
		aqActiveShape = aqProposedActiveShape;
		
	facet->ChangeActiveShape(ev, aqActiveShape, NULL);	
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AdjustUsedShape
//----------------------------------------------------------------------------------------
//	suggestedUsedShape is the frame shape. The default implementation returns NULL because
//	we don't want to maintain our own used shape.

ODShape* FW_CFrame::AdjustUsedShape(Environment* ev, ODShape* suggestedUsedShape)
{
FW_UNUSED(ev);
FW_UNUSED(suggestedUsedShape);
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AdjustActiveShape
//----------------------------------------------------------------------------------------
//	suggestedActiveShape is the used shape. The default implementation returns NULL because
//	we don't want to maintain an active shape

ODShape* FW_CFrame::AdjustActiveShape(Environment* ev, ODFacet* facet, ODShape* suggestedActiveShape)
{
FW_UNUSED(ev);
FW_UNUSED(facet);
FW_UNUSED(suggestedActiveShape);
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::RequestFrameShape
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::RequestFrameShape(Environment *ev, ODShape* shape, ODCanvas* biasCanvas)
{
	FW_CAcquiredODShape aqOldFrameShape = FW_CopyAndRelease(ev, AcquireFrameShape(ev, biasCanvas));
	
	FW_CAcquiredODShape aqNewFrameShape = fODFrame->RequestFrameShape(ev, shape, biasCanvas);
	
	FW_Boolean changed = !aqNewFrameShape->IsSameAs(ev, aqOldFrameShape);
	
	if (changed)
		FrameShapeChanged(ev); 
	
	return changed;
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::FacetAdded
//----------------------------------------------------------------------------------------
//	When overridden call inherited::FacetAdded FIRST

void FW_CFrame::FacetAdded(Environment* ev, ODFacet* facet, unsigned short facetCount)
{		
	// ----- Install the newest facet as the active facet of this frame
	this->SetActiveFacet(ev, facet);
	
	// ----- Create the window wrapper -----
	// ----- We have to wait for facet added because 
	// ----- at FrameAdded it is too early
	FW_CAcquiredODWindow aqODWindow = fODFrame->AcquireWindow(ev);
	this->PrivCreateWindow(ev, aqODWindow);	// Might not create a FW_CWindow if already has one	

	// ----- for floating windows: remove all the foci by setting an empty set.
	if (aqODWindow->IsFloating(ev))
	{
		GetPresentation(ev)->AdoptFocusSet(ev, NULL);
	}
	
	// ----- Create the Shadow Window for Windows
#ifdef FW_BUILD_WIN
	CreateShadowWindow(ev);
#endif	

	// ----- First facet: update used shape and create the sub-views
	if (facetCount == 1)
	{
		UpdateUsedShape(ev);
		
		// Create views only once during the frame's life!
		if (!fSubViewsCreated)
		{
			if (fResourceId != 0)
			{
				// Automatic view creation from resources
				CreateSubViewsFromResource(ev, fResourceId);
			}
			else
			{
				// Manual view creation
				CreateSubViews(ev);
			}
			fSubViewsCreated = TRUE;
		}
		
		if (fInternalTransformRealigmentNeeded)
			PrivRealignInternalTransform(ev);
	}

	// ----- Give my frame a chance to update the active shape of the facet -----
	UpdateActiveShape(ev, facet);
	
	// ----- Finally invalidate the content of the facet so it will be redrawn -----
	FW_CAcquiredODShape aqUsedShape = AcquireUsedShape(ev); 
	facet->Invalidate(ev, aqUsedShape, NULL);
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::FacetRemoved
//----------------------------------------------------------------------------------------
//	When overridden call inherited::FacetRemoved last

void FW_CFrame::FacetRemoved(Environment* ev, ODFacet* odFacet, unsigned short facetCount)
{
	if (this->GetActiveFacet(ev) == odFacet)
	{
		FW_CFrameFacetIterator iter(ev, this);
		this->SetActiveFacet(ev, iter.First(ev));
	}
	
	// ----- Delete the fWindow if it was the last facet -----
	if (facetCount == 1)
	{
		FW_ASSERT(fWindow);
		
		if (!fWindow->IsFloating(ev))	// If floating window the part has to release
			delete fWindow;
		
		fWindow = NULL;
	}
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetWindow
//----------------------------------------------------------------------------------------

FW_CWindow* FW_CFrame::GetWindow(Environment* ev) const
{
FW_UNUSED(ev);
	return fWindow;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ViewTypeChanged
//----------------------------------------------------------------------------------------

void FW_CFrame::ViewTypeChanged(Environment* ev, 
								ODTypeToken newViewType, 
								ODTypeToken oldViewType)
{
FW_UNUSED(oldViewType);
	Invalidate(ev);
	
	if (fViewAs != NULL)
		fPresentation->PrivReleaseViewAs(fViewAs);
	fViewAs = NULL;
	
	PrivCreateViewAs(ev);
		
	UpdateUsedAndActiveShapes(ev);
	
	Invalidate(ev);
	
	// ----- Only If I have an associated FW_MDroppableFrame -----
	if (fDroppable)
		fODFrame->SetDroppable(ev, FW_MDroppableFrame::PrivDroppableStateToBoolean(fDroppableState, newViewType));
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::BuildThumbnail
//----------------------------------------------------------------------------------------
//	By default does nothing

void FW_CFrame::BuildThumbnail(Environment* ev, FW_CGraphicContext& gc, FW_CRect& usedRect)
{
FW_UNUSED(ev);
FW_UNUSED(gc);
FW_UNUSED(usedRect);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetIcon
//----------------------------------------------------------------------------------------
//	By default returns the part icon

FW_CIcon FW_CFrame::GetIcon(Environment* ev, short iconSize) const
{
	FW_CSharedLibraryResourceFile resourceFile(ev, fPart->GetPartInstance(ev));
	FW_CIcon icon(resourceFile, fPart->GetViewAsIconID(ev), iconSize);
	
	return icon;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PresentationChanged
//----------------------------------------------------------------------------------------

void FW_CFrame::PresentationChanged(Environment* ev)
{	
FW_UNUSED(ev);
	SetViewId(ev, fPresentation->GetPresentationType(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SequenceChanged
//----------------------------------------------------------------------------------------

void FW_CFrame::SequenceChanged(Environment* ev)
{	
FW_UNUSED(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleActivateEvent
//----------------------------------------------------------------------------------------

void FW_CFrame::HandleActivateEvent(Environment* ev, const FW_CActivateEvent& theActivateEvent)
{
	if (theActivateEvent.IsActivating(ev))
	{
		if (fNeedsFoci) 
			PrivActivateFrame(ev, theActivateEvent.GetFacet(ev));
	}
	else
	{
		fNeedsFoci = fHasSelectionFocus;
	}
	
	FW_CSuperView::HandleActivateEvent(ev, theActivateEvent);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleSuspendResumeEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HandleSuspendResumeEvent(Environment* ev, const FW_CSuspendResumeEvent& theResumeSuspendEvent)
{
	FW_Boolean backToForeground = !theResumeSuspendEvent.IsGoingToBackground(ev);
	
	// [LSD] see later to use keyFocus and menuFocus instead for target management
	if (HasSelectionFocus(ev))
	{
		if (backToForeground) {
			GetTarget(ev)->ActivateTarget(ev);
		} else {
			GetTarget(ev)->DeactivateTarget(ev);
		}
	}

	if (GetWindow(ev)->IsFloating(ev) && IsRoot(ev))
	{
		ODWindowState* windowState = fPart->GetSession(ev)->GetWindowState(ev);
		// ----- We have to test that this frame belongs to the part with the selection focus
		ODArbitrator* arbitrator =  fPart->GetSession(ev)->GetArbitrator(ev);
		FW_CAcquiredODFrame aqODFrame = arbitrator->AcquireFocusOwner(ev, FW_CPart::gSelectionFocusToken);
		if (aqODFrame != NULL)
		{
			FW_CAcquiredODPart aqODPart = aqODFrame->AcquirePart(ev);
			if (aqODPart == fPart->GetODPart(ev))
			{
				ODWindowState* windowState = fPart->GetSession(ev)->GetWindowState(ev);
				FW_CFloatingWindow::HideShowFloatingWindows(ev, windowState, backToForeground);
			}
		}
	}
	
	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleMouseEnter
//----------------------------------------------------------------------------------------
// where is in frame coordinate

FW_Boolean FW_CFrame::HandleMouseEnter(Environment *ev, ODFacet* odFacet, const FW_CPoint& where)
{
	fViewUnderCursor = GetViewContaining(ev, odFacet, where);
	if (fViewUnderCursor == NULL)
		return FALSE;

	if (!fViewUnderCursor->AdjustCursor(ev, odFacet, where))
		FW_gArrowCursor.Select();

	return (fViewUnderCursor->IsEnabled(ev)) ? 
		fViewUnderCursor->DoMouseEnter(ev, odFacet, where) : FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleMouseLeave
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HandleMouseLeave(Environment *ev, ODFacet* odFacet)
{
	FW_gArrowCursor.Select();
	
	FW_Boolean result = fViewUnderCursor ? fViewUnderCursor->DoMouseLeave(ev, odFacet) : FALSE;

	fViewUnderCursor = NULL;
	
	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleMouseWithin
//----------------------------------------------------------------------------------------
// where is in frame coordinate

FW_Boolean FW_CFrame::HandleMouseWithin(Environment *ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
	FW_CView* viewUnderCursor = GetViewContaining(ev, odFacet, theMousePoint);
	if (viewUnderCursor == NULL)
		return FALSE;
		
	if (viewUnderCursor != fViewUnderCursor)
	{
		if (fViewUnderCursor != NULL)
			fViewUnderCursor->DoMouseLeave(ev, odFacet);
		
		viewUnderCursor->DoMouseEnter(ev, odFacet, theMousePoint);
	}
	else
		viewUnderCursor->DoMouseWithin(ev, odFacet, theMousePoint);
		
	fViewUnderCursor = viewUnderCursor;
	
	if (!fViewUnderCursor->AdjustCursor(ev, odFacet, theMousePoint))
		FW_gArrowCursor.Select();
		
	return TRUE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GeometryChanged
//----------------------------------------------------------------------------------------
//	the geometry (clip shape or external transform) of one of my facet or one of my
//	embedding facet as changed

void FW_CFrame::GeometryChanged(Environment *ev,
								ODFacet* odFacet,
								FW_Boolean clipShapeChanged,
								FW_Boolean externalTransformChanged)
{
FW_UNUSED(ev);
FW_UNUSED(odFacet);
FW_UNUSED(clipShapeChanged);
FW_UNUSED(externalTransformChanged);

#ifdef FW_BUILD_WIN
	MoveSizeShadowWindow(ev);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HandleDraw
//----------------------------------------------------------------------------------------
// invalidShape is in frame coordinate

FW_Boolean FW_CFrame::HandleDraw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape)
{
	// 	if the facet doesn't have a graphicdevice it is not one of my facet anymore
	//	That means that FacetRemoved has been called but my container is still trying
	//	do draw me.[HLX] Might be a bug in OpenDoc. Might have been fixed in DR4
	if (FW_GetFacetGraphicDevice(ev, odFacet) == NULL)
		return TRUE;
		
#ifdef FW_BUILD_WIN
	// [HLX] bug in windows? the invalidShape is in pixels instead of points
	aqInvalidShape->Outset(ev, ff(1));	// [HLX] Because of rounding errors
	FW_CGraphicDevice *device = FW_GetFacetGraphicDevice(ev, odFacet);
	device->PixelToPoint(ev, aqInvalidShape);
#endif
	
	// ----- Check if the fViewAs has been created
	PrivCreateViewAs(ev);
	if (fViewAs == NULL)
	{
		FW_CSuperView::HandleDraw(ev, odFacet, invalidShape);
		
		// ----- If hilited state draw the hilite state
		if (odFacet->GetHighlight(ev) == kODFullHighlight)
			DrawHighlight(ev, odFacet, kODFullHighlight, invalidShape);
	}
	else
	{
		fViewAs->Draw(ev, this, odFacet, invalidShape);
	}

	return TRUE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::Draw
//----------------------------------------------------------------------------------------
// invalidShape is in content coordinate

void FW_CFrame::Draw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape)
{
FW_UNUSED(ev);
FW_UNUSED(odFacet);
FW_UNUSED(invalidShape);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::DrawHighlight
//----------------------------------------------------------------------------------------

void FW_CFrame::DrawHighlight(Environment *ev, ODFacet* facet, ODHighlight hilite, ODShape* invalidShape)
{
	FW_CFrameContext fc(ev, facet, invalidShape);
	FW_CAcquiredODShape usedShape = AcquireUsedShape(ev);
	if (hilite == kODFullHighlight)
		FW_CRegionShape::RenderRegion(fc, usedShape, FW_kFill, FW_CInk(FW_kHilite));
	else if (hilite == kODNoHighlight || hilite == kODDimHighlight)
	{
#ifdef FW_BUILD_MAC
		FW_CRegionShape::RenderRegion(fc, usedShape, FW_kFill, FW_CInk(FW_kHilite));	
#endif
#ifdef FW_BUILD_WIN
		facet->Invalidate(ev, NULL, NULL);	// NULL for the shape is fine
#endif	
	}
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HighlightChanged
//----------------------------------------------------------------------------------------

void FW_CFrame::HighlightChanged(Environment *ev, ODFacet* facet)
{
	// In the Icon/Thumbnail case just invalidate the facet
	if (GetViewType(ev) != FW_CPart::gViewAsFrameToken)
		facet->Invalidate(ev, NULL, NULL);	// NULL for the shape is fine
	else
		DrawHighlight(ev, facet, facet->GetHighlight(ev), NULL);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AdjustCursor
//----------------------------------------------------------------------------------------
// where is in frame coordinate

FW_Boolean FW_CFrame::AdjustCursor(Environment *ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
FW_UNUSED(ev);
FW_UNUSED(odFacet);
FW_UNUSED(theMousePoint);
	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetODPart
//----------------------------------------------------------------------------------------

ODPart* FW_CFrame::GetODPart(Environment* ev) const
{
	return fPart->GetODPart(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetContentView
//----------------------------------------------------------------------------------------

void FW_CFrame::SetContentView(Environment* ev, FW_CSuperView* view) 
{
	// SetContentView can only be called once to changed fContentView
	// which was initialized to the frame itself
	FW_ASSERT(IsContentView(ev) == TRUE);
	
	if (view != this) 
	{
		PrivSetIsContentView(ev, FALSE);
		fContentView = view;

		// Make the content view the default target for menu and keyboard events
		fTarget = fContentView;

		FW_CRect bounds = view->GetBounds(ev);
		FW_CSuperView* superView = view->GetSuperView(ev);
		if (superView)
			superView->ViewContentToFrame(ev, bounds);

		PrivContentViewLocationChanged(ev, bounds.TopLeft());
	}
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AdoptScroller
//----------------------------------------------------------------------------------------

void FW_CFrame::AdoptScroller(Environment* ev, FW_CScroller* scroller)
{
	// [LSD] Set the content view's scrolling flag in case it wasn't set in ctor
	//		 To be reviewed.
	FW_CSuperView* contentView = GetContentView(ev);  // could be the frame itself
	if (contentView->IsScrolling(ev) == false)
	{
		contentView->SetScrollingInX(ev, true);
		contentView->SetScrollingInY(ev, true);
	}
	
#ifdef FW_DEBUG
	scroller->PrivCheckFrame(ev);
#endif		
	
	// [LSD] don't allow to change an existing scroller for now
	FW_ASSERT(fScroller == NULL);
	fScroller = scroller;
	
	PrivUpdateScrollParameters(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ExternalizeFrame
//----------------------------------------------------------------------------------------

void FW_CFrame::ExternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView)
{
	// ----- Write first a version number
	FW_CByteArray byteArray(&FW_kFrameVersion, sizeof(FW_FrameVersion));
	storageUnitView->SetValue(ev, byteArray);

	// ----- Write content view offset
	byteArray.Set(&fContentViewOffset, sizeof(fContentViewOffset));
	storageUnitView->SetValue(ev, byteArray);

	// ----- Write requestedFrame flag
	byteArray.Set(&fRequestedFrame, sizeof(fRequestedFrame));
	storageUnitView->SetValue(ev, byteArray);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::InternalizeFrame
//----------------------------------------------------------------------------------------

void FW_CFrame::InternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView)
{
	FW_CByteArray byteArray;

	// ----- Read first a version number
	FW_FrameVersion version;
	storageUnitView->GetValue(ev, sizeof(FW_FrameVersion), byteArray);
	byteArray.CopyBuffer(&version, sizeof(FW_FrameVersion));
	FW_ASSERT(version == FW_kFrameVersion);
	
	// ----- Read content view offset
	storageUnitView->GetValue(ev, sizeof(fContentViewOffset), byteArray);
	byteArray.CopyBuffer(&fContentViewOffset, sizeof(fContentViewOffset));
	
	// ----- Read requestedFrame flag
	storageUnitView->GetValue(ev, sizeof(fRequestedFrame), byteArray);
	byteArray.CopyBuffer(&fRequestedFrame, sizeof(fRequestedFrame));
	
	// ----- The Internal transform needs to be realign because
	// the view layout may have changed
	fInternalTransformRealigmentNeeded = TRUE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetContentExtent
//----------------------------------------------------------------------------------------

void FW_CFrame::GetContentExtent(Environment* ev, FW_CPoint& extent) const
{
	extent = GetContentView(ev)->GetExtent(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireContentShape
//----------------------------------------------------------------------------------------
//	return the content shape in Frame coordinates

ODShape* FW_CFrame::AcquireContentShape(Environment *ev)
{	
	FW_CSuperView* contentView = GetContentView(ev);
	
	FW_CRect contentRect(contentView->GetBounds(ev));
	
	FW_CSuperView* superView = contentView->GetSuperView(ev);
	if (superView)
		superView->ViewContentToFrame(ev, contentRect);
	
	return FW_NewODShape(ev, contentRect);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFocusSet
//----------------------------------------------------------------------------------------

ODFocusSet* FW_CFrame::GetFocusSet(Environment* ev) const
{
	return fPresentation->GetFocusSet(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::BeginRelinquishFocus
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::BeginRelinquishFocus(Environment *ev, ODTypeToken focus, ODFrame* proposedFrame)
{
FW_UNUSED(ev);
FW_UNUSED(focus);
FW_UNUSED(proposedFrame);
	return TRUE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AbortRelinquishFocus
//----------------------------------------------------------------------------------------

void FW_CFrame::AbortRelinquishFocus(Environment *ev, ODTypeToken focus, ODFrame* proposedFrame)
{
FW_UNUSED(ev);
FW_UNUSED(focus);
FW_UNUSED(proposedFrame);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ClonePartInfo
//----------------------------------------------------------------------------------------

void FW_CFrame::ClonePartInfo(Environment* ev, 
							ODDraftKey key,
							ODInfoType partInfo,
							ODStorageUnitView* storageUnitView,
							ODFrame* scope)
{
FW_UNUSED(ev);
FW_UNUSED(key);
FW_UNUSED(partInfo);
FW_UNUSED(storageUnitView);
FW_UNUSED(scope);
	// Don't do anything for now
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CFrame::CreateShadowWindow
//----------------------------------------------------------------------------------------
// Create a transparent Window so that we get a chance to handle messages sent by Windows
// controls to their parent Window.

void FW_CFrame::CreateShadowWindow(Environment *ev)
{
	ODPlatformWindow parentHWnd = fWindow->GetPlatformWindow(ev);
#ifdef FW_BUILD_WIN16
	HINSTANCE hInstance = (HINSTANCE)GetWindowWord(parentHWnd, GWW_HINSTANCE);
#endif
#ifdef FW_BUILD_WIN32
	HINSTANCE hInstance = (HINSTANCE)GetWindowLong(parentHWnd, GWL_HINSTANCE);
#endif
	
	WNDCLASS wc;
	
	if (!GetClassInfo(hInstance, "ShadowWindow", &wc))
	{
		wc.style = CS_DBLCLKS;
		wc.lpfnWndProc = ShadowWindowProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 4;
		wc.hInstance = hInstance;
		wc.hIcon = NULL;
		wc.hCursor = LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground = NULL;
		wc.lpszMenuName = NULL;
		wc.lpszClassName = "ShadowWindow";
		
		if (!RegisterClass(&wc))
			return;
	}
	
	fShadowWindow = CreateWindowEx(WS_EX_TRANSPARENT, "ShadowWindow", "",
								   WS_CHILD | WS_VISIBLE, 0, 0, 0, 0,
								   parentHWnd, NULL, hInstance, (LPSTR) this);
	MoveSizeShadowWindow(ev);
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CFrame::MoveSizeShadowWindow
//----------------------------------------------------------------------------------------

void FW_CFrame::MoveSizeShadowWindow(Environment* ev)
{
	if (fShadowWindow != NULL)
	{
		ODFacet* facet = GetActiveFacet(ev);
		FW_CAcquiredODShape aqShape(FW_CopyAndRelease(ev, facet->AcquireActiveShape(ev, NULL)));
		FW_CAcquiredODTransform aqExternalTransform = facet->AcquireExternalTransform(ev, NULL);
		aqShape->Transform(ev, aqExternalTransform);
		
		FW_CRect bBox;
		aqShape->GetBoundingBox(ev, &bBox);
		
		// Shape bounding box is in 72dpi, need to convert to pixels
		if(!IsRoot(ev))
		{
			// [KVV] ??? This is only necessary for non-root frames
		
			HDC hDC = ::GetDC(NULL);
			int xRes = ::GetDeviceCaps(hDC, LOGPIXELSX);
			int yRes = ::GetDeviceCaps(hDC, LOGPIXELSY);
			::ReleaseDC(NULL, hDC);
			
			FW_Fixed xScale = FW_FixDiv(ff(xRes), ff(72));
			FW_Fixed yScale = FW_FixDiv(ff(yRes), ff(72));
			
			bBox.left 	= FW_FixMul(bBox.left,	xScale);
			bBox.top 	= FW_FixMul(bBox.top,	yScale);
			bBox.right 	= FW_FixMul(bBox.right,	xScale);
			bBox.bottom	= FW_FixMul(bBox.bottom,yScale);
		}
		
		FW_SPlatformRect wndRect = bBox.AsPlatformRect();
		
		::MoveWindow(fShadowWindow, wndRect.left, wndRect.top,
				   	wndRect.right - wndRect.left, wndRect.bottom - wndRect.top, TRUE);
	}
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CFrame::ShadowWindowProc
//----------------------------------------------------------------------------------------

LRESULT CALLBACK FW_CFrame::ShadowWindowProc(HWND hWnd, UINT msg,
											 WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
#ifdef FW_DEBUG
		// ----- Frame the shadow window for ease of debugging
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC hDC = ::BeginPaint(hWnd, &ps);
				
				if(::GetPrivateProfileInt("Debug", "ShadowWindow Outline", FALSE, "ODF.ini"))
				{
					static const char pattern[8][2] = 
					{
						{ 0xAA, 0x00 },
						{ 0x55, 0x00 },
						{ 0xAA, 0x00 },
						{ 0x55, 0x00 },
						{ 0xAA, 0x00 },
						{ 0x55, 0x00 },
						{ 0xAA, 0x00 },
						{ 0x55, 0x00 }
					};
					
					HBITMAP hBitmap = ::CreateBitmap(8, 8, 1, 1, pattern);
	
					HBRUSH hBrush = ::CreatePatternBrush(hBitmap);
	
					::SetTextColor(hDC, RGB(0xFF, 0x00, 0xFF));
					
					RECT rc;
					::GetClientRect(hWnd, &rc);
					
					HRGN hRgn = ::CreateRectRgnIndirect(&rc);
					
					::FrameRgn(hDC, hRgn, hBrush, 3, 3);
					
					::DeleteObject(hRgn);
					::DeleteObject(hBrush);
					::DeleteObject(hBitmap);
				}

				::EndPaint(hWnd, &ps);
			}
			return 0;
#endif

		case WM_CREATE:
			SetWindowLong(hWnd, 0, (long) ((LPCREATESTRUCT) lParam)->lpCreateParams);
			return 0;
			
		case WM_NCHITTEST:
			return HTTRANSPARENT;
			
		case WM_COMMAND:
			switch (HIWORD(wParam))
			{
				case BN_CLICKED:
					{
						HWND hButton = HWND(lParam);
						
						// If this is a window
						if (hButton != NULL && ::IsWindow(hButton))
						{
							// Get the scroll bar helper
							FW_CPrivWinButtonHelper* buttonHelper =
								FW_DYNAMIC_CAST(FW_CPrivWinButtonHelper, FW_CPrivWinControlHelper::HWNDToHelper(hButton));
		
							// Tell the button to update itself and the pane
							buttonHelper->HandleButtonMessage();
							return 0L;
						}
					}
					break;
			}
			break;
			
		case WM_HSCROLL:
		case WM_VSCROLL:
			{
				HWND hBar = HWND(lParam);

				// If this is not a scroll bar control, but a Windows
				// window scroll bar, pass it to Windows wndproc
				if (hBar == NULL)
				{
					FW_DEBUG_MESSAGE("Warning: ODF does not use Window scroll bars!");
					break;
				}

				// If this is a window
				if (hBar != NULL && ::IsWindow(hBar))
				{
					// Get the scroll bar helper
					FW_CPrivWinScrollBarHelper* sbHelper =
						FW_DYNAMIC_CAST(FW_CPrivWinScrollBarHelper, FW_CPrivWinControlHelper::HWNDToHelper(hBar));

					// Tell the scroll bar to update itself and the pane
					if (sbHelper != NULL)
					{
						sbHelper->HandleScrollMessage(LOWORD(wParam), HIWORD(wParam));
						return 0L;
					}
				}
				break;
			}
	}
	
	return DefWindowProc(hWnd, msg, wParam, lParam);
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CFrame::HasModalFocus
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HasModalFocus(Environment* ev) const
{
	FW_CAcquiredODFrame aqODFrame = fPart->GetSession(ev)->GetArbitrator(ev)->AcquireFocusOwner(ev, FW_CPart::gModalFocusToken);
	return aqODFrame == fODFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HasPropertyOnClipboard
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HasPropertyOnClipboard(Environment* ev, ODPropertyName propertyName, ODValueType type)
{
	ODSession* session = GetPart(ev)->GetSession(ev);
	ODArbitrator* arbitrator = session->GetArbitrator(ev);
	FW_CAcquiredODFrame aqODFrame = arbitrator->AcquireFocusOwner(ev, FW_CPart::gClipboardFocusToken);
	
	if ((aqODFrame == fODFrame) || (arbitrator->RequestFocus(ev, FW_CPart::gClipboardFocusToken, fODFrame)) )
	{	
		ODStorageUnit* su = session->GetClipboard(ev)->GetContentStorageUnit(ev);
		return su->Exists(ev, propertyName, type, 0);
	}
	else
	{
		return FALSE;
	}
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ChangeInternalTransform
//----------------------------------------------------------------------------------------

void FW_CFrame::ChangeInternalTransform(Environment* ev, ODTransform* internalTransform, ODCanvas* biasCanvas)
{
	// *LSD2  hack can be removed after DR4 ?
#ifdef FW_BUILD_MAC
	// [HLX] Temporary because of a bug in OpenDoc
	//	ChangeInternalTransform changes the clip and the origin and does not restore it
	GrafPtr port;
	::GetPort(&port);	
	Point origin;
	origin.h = port->portRect.left;
	origin.v = port->portRect.top;
	ODRgnHandle clipRgn = ::NewRgn();
	::GetClip(clipRgn);
#endif
	
	fODFrame->ChangeInternalTransform(ev, internalTransform, biasCanvas);

#ifdef FW_BUILD_MAC
	// [HLX] Temporary because of a bug in OpenDoc
	::SetPort(port);
	::SetClip(clipRgn);
	::DisposeRgn(clipRgn);
	::SetOrigin(-origin.h, -origin.v);
#endif
	
	// Now invalidate all the views Internal transforms
	PrivInvalidateViewIternalTransform(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ContainingPartPropertiesUpdated
//----------------------------------------------------------------------------------------

void FW_CFrame::ContainingPartPropertiesUpdated(Environment *ev, ODStorageUnit* propertyUnit)
{
FW_UNUSED(ev);
FW_UNUSED(propertyUnit);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::NewPrintHandler
//----------------------------------------------------------------------------------------

FW_CPrintHandler* FW_CFrame::NewPrintHandler(Environment* ev)
{
FW_UNUSED(ev);

	return NULL;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::SetLocation
//----------------------------------------------------------------------------------------

void FW_CFrame::SetLocation(Environment* ev, const FW_CPoint& location, FW_Boolean redraw)
{
FW_UNUSED(ev);
FW_UNUSED(location);
FW_UNUSED(redraw);

	FW_DEBUG_MESSAGE("Can't call SetLocation on a FW_CFrame");
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::RelinquishFocusSet
//----------------------------------------------------------------------------------------
//	Usually should not have to be overriden. Can be overriden for special case

void FW_CFrame::RelinquishFocusSet(Environment *ev)
{
	ODArbitrator* arbitrator = fPart->GetSession(ev)->GetArbitrator(ev);
	arbitrator->RelinquishFocusSet(ev, GetFocusSet(ev), GetODFrame(ev));
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::RequestFocusSet
//----------------------------------------------------------------------------------------
//	Usually should not have to be overriden. Can be overriden for special case

FW_Boolean FW_CFrame::RequestFocusSet(Environment *ev)
{
	ODArbitrator* arbitrator = fPart->GetSession(ev)->GetArbitrator(ev);
	return arbitrator->RequestFocusSet(ev, GetFocusSet(ev), GetODFrame(ev));
}

//----------------------------------------------------------------------------------------
// FW_CFrame::HandleIdle
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::HandleIdle(Environment* ev, const FW_CNullEvent& theNullEvent)
{
	if (IsInLimbo(ev)) return false;		// don't idle if we're in limbo

	return DoIdle(ev, theNullEvent);	
}

//========================================================================================
// Implementation Only API
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivCreateWindow
//----------------------------------------------------------------------------------------

FW_CWindow* FW_CFrame::PrivCreateWindow(Environment* ev, ODWindow* odWindow)
{
	FW_ASSERT(odWindow);
//	FW_ASSERT(fWindow == NULL || fWindow->GetID(ev) == odWindow->GetID(ev));
	
	// ----- Test first if this frame was in another window -----
	if (fWindow != NULL && fWindow->GetID(ev) != odWindow->GetID(ev))
	{
		delete fWindow;
		fWindow = NULL;
	}
	
	if (fWindow == NULL)
	{
		ODPlatformWindow platformWindow = odWindow->GetPlatformWindow(ev);
		
		FW_CWindow* fwWindow = NULL;
		
		if (IsRoot(ev))
		{
#ifdef FW_BUILD_MAC
			fwWindow = (FW_CWindow*) ::GetWRefCon(platformWindow);
#endif

#ifdef FW_BUILD_WIN
			fwWindow = (FW_CWindow*) ::GetProp(platformWindow, "ODF:Window");
#endif
		}
		
		if (fwWindow != NULL)
			fWindow = fwWindow;
		else
			fWindow = new FW_CWindow(ev, fPart, odWindow->GetID(ev));
	}

	fInSharedWindow = IsRoot(ev) && fWindow->IsFloating(ev);
	
	return fWindow;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivSetDroppable
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivSetDroppable(Environment *ev, FW_MDroppableFrame* droppable, FW_Boolean isDroppable)
{
	fDroppable = droppable;
	fODFrame->SetDroppable(ev, isDroppable);
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivMacInGrowBox
//----------------------------------------------------------------------------------------
 
 void FW_CFrame::PrivMacInGrowBox(Environment* ev, const FW_CMacWindowEvent& windowEvent)
 {	
	FW_CRect growLimits(FW_IntToFixed(64), FW_IntToFixed(64), FW_IntToFixed(32000), FW_IntToFixed(32000));
	this->AdjustWindowGrowLimits(ev, growLimits[FW_kTopLeft], growLimits[FW_kBotRight]);
	
	// If minimum is lareger than maximun then do something
	if (growLimits.left > growLimits.right)
		growLimits.left = growLimits.right;
	if (growLimits.top > growLimits.bottom)
		growLimits.top = growLimits.bottom;
					
 	ODEventData* eventData = ((FW_CMacWindowEvent&)windowEvent).GetPlatformEvent();
 	FW_CAcquiredODWindow aqODWindow = fWindow->AcquireODWindow(ev);
	
	FW_PlatformError error;
	FW_Boolean result = FW_PrivMacTrackResizeWindow(aqODWindow->GetPlatformWindow(ev), 
 									growLimits,
 									eventData->where,
 									&error);
 	FW_FailOnError(error);
 	if (result)
 	{
		aqODWindow->AdjustWindowShape(ev);
	}
}
#endif

//----------------------------------------------------------------------------------------
// 	FW_CFrame::PrivActivateFrame
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivActivateFrame(Environment *ev, ODFacet* activatedFacet)
{ 
	if (CanBeActiveFrame(ev))
	{
		SetActiveFacet(ev, activatedFacet);
		
		if (fHasSelectionFocus == FALSE)
		{
			if (RequestFocusSet(ev))
			{
				fNeedsFoci = FALSE;
				
				FW_CFocusSetIterator iter(ev, this);
				for( ODTypeToken token=iter.First(ev); iter.IsNotComplete(ev); token=iter.Next(ev) ) {
					FocusStateChanged(ev, token, TRUE, NULL);			
				}
			}
		}
	}
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivGetFrameShapeBounds
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivGetFrameShapeBounds(Environment* ev, FW_CRect& rect) const
{
	FW_CAcquiredODShape aqODShape = AcquireFrameShape(ev, NULL);
	rect = FW_GetShapeBoundingBox(ev, aqODShape);
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::PrivFacetRemoved
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivFacetRemoved(Environment* ev, ODFacet* facet)
{
	FW_ASSERT(fFacetCount > 0);
	
	FacetRemoved(ev, facet, fFacetCount);
	
	// ----- One less facet -----
	fFacetCount--;
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::PrivFacetAdded
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivFacetAdded(Environment* ev, ODFacet* facet)
{
	// ----- One more facet -----
	fFacetCount++;	

	FacetAdded(ev, facet, fFacetCount);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivAttachSourceFrame
//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivAttachSourceFrame should not be called if sourceFrame == NULL

void FW_CFrame::PrivAttachSourceFrame(Environment* ev, FW_CFrame* sourceFrame)
{
	FW_ASSERT(sourceFrame != NULL);
	FW_ASSERT(fSourceFrame == this);	// fSourceFrame is set to this in the constructor
	
	fSourceFrame = sourceFrame->GetSourceFrame(ev);
}

//----------------------------------------------------------------------------------------
// 	FW_CFrame::PrivActiveWindowOnMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFrame::PrivActiveWindowOnMouseDown(Environment* ev, ODFacet* facet, FW_Boolean clickedInFrame)
{
	if (!GetWindow(ev)->IsFloating(ev))
	{
		FW_Boolean windowUnselected = !GetWindow(ev)->IsActive(ev);
		
		// ----- Select the window if not already selected -----
		if (windowUnselected)
			GetWindow(ev)->Select(ev);		
		
		// ----- active frame on mousedown only if clicked in the frame -----
		if (clickedInFrame == TRUE && fHasSelectionFocus == FALSE)
			PrivActivateFrame(ev, facet);
		
		// ----- If the window was unselected and I don't want the ----- 
		// ----- first click just return TRUE;
		if (windowUnselected && !GetFrontClicks(ev))
				return TRUE;
	}
	
	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivUpdateScrollParameters
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivUpdateScrollParameters(Environment* ev, FW_Boolean notify)
{
	if (fScroller && !fInternalTransformRealigmentNeeded)
		fScroller->UpdateScrollParameters(ev, notify);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivContentViewLocationChanged
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivContentViewLocationChanged(Environment* ev, const FW_CPoint& newLocation)
{
	if (fContentViewOffset != newLocation)
	{
		FW_CAcquiredODTransform aqFrameInternalTransform = AcquireInternalTransform(ev);
		aqFrameInternalTransform->MoveBy(ev, (ODPoint*)&(-fContentViewOffset));
		aqFrameInternalTransform->MoveBy(ev, (ODPoint*)&newLocation);
		ChangeInternalTransform(ev, aqFrameInternalTransform);
		
		fContentViewOffset = newLocation;
	}
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivRealignInternalTransform
//----------------------------------------------------------------------------------------
//	This is call when a document is reopened. Because the internal transform is saved by OpenDoc
//	and the InternalTransform contains the offset of the ContentView, if this offset is not
//	the same when the document is reopened I need to realign the Internal transform
void FW_CFrame::PrivRealignInternalTransform(Environment* ev)
{
	FW_ASSERT(fContentView);
	
	FW_CRect bounds = fContentView->GetBounds(ev);
	FW_CSuperView* contentViewParent = fContentView->GetSuperView(ev);
	if (contentViewParent)
		contentViewParent->ViewContentToFrame(ev, bounds);
	
	PrivContentViewLocationChanged(ev, bounds.TopLeft());
	
	fInternalTransformRealigmentNeeded = FALSE;

	PrivUpdateScrollParameters(ev, FALSE);	// Don't want to notify
}

//---------------------------------------------------------------------------------------
//	FW_CFrame::PrivDragging
//---------------------------------------------------------------------------------------

void FW_CFrame::PrivDragging(Environment *ev, FW_Boolean state)
{
FW_UNUSED(ev);
FW_UNUSED(state);
	// No embedded frames
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivCreateSubViewsFromStream
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivCreateSubViewsFromStream(Environment* ev, FW_CReadableStream& stream)
{
	FW_PREREGISTER_RUNTIME_OBJECT(stream, 
								this, 
								FW_kPreregisteredFrameObject);
								
	FW_CSuperView::PrivCreateSubViewsFromStream(ev, stream);

	short count;
	stream >> count;
	FW_ASSERT(count >= 0);
	FW_ASSERT(count <= 1);
	if (count == 1)
	{
		FW_CPrivBaseScroller* scroller;
		FW_READ_DYNAMIC_OBJECT(stream, &scroller, FW_CPrivBaseScroller);
		
		// [HLX] I am always dealing with a FW_CScroller
		AdoptScroller(ev, (FW_CScroller*)scroller);	
	}
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivButtonAttached
//----------------------------------------------------------------------------------------

void FW_CFrame::PrivButtonAttached(Environment* ev, FW_CButton* button)
{
FW_UNUSED(ev);
FW_UNUSED(button);
	// Nothing to do look at FW_CDialogFrame
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivCreateViewAs
//----------------------------------------------------------------------------------------
// Call in places where the viewas object might not be yet created 

void FW_CFrame::PrivCreateViewAs(Environment* ev)
{
	ODTypeToken viewType = GetViewType(ev);
	if (viewType != FW_CPart::gViewAsFrameToken && fViewAs == NULL)
	{
		fViewAs = fPresentation->PrivAcquireViewAs(ev, this, viewType);
	}
}
