//========================================================================================
//
//	File:				FWPresen.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWVIEWAS_H
#include "FWViewAs.h"
#endif

#ifndef FWEXCEPT_H
#include "FWExcept.h"
#endif

// ----- OS Layer -----

#ifndef FWODGEOM_H
#include "FWODGeom.h"
#endif

#ifndef FWWINDOW_H
#include "FWWindow.h"
#endif

// ----- Foundation Includes -----

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif

// ----- OpenDoc Includes -----
#ifndef SOM_ODArbitrator_xh
#include <Arbitrat.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#ifndef SOM_ODFocusSet_xh
#include <FocusSet.xh>
#endif

#ifndef SOM_ODShape_xh
#include <Shape.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#if defined(__MWERKS__) && GENERATING68K
// A hack to work around a bug
#pragma import list somGetGlobalEnvironment
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment fwpart2
#endif

//========================================================================================
//	Template Instantiations
//========================================================================================

FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollectionIterator, FW_CPresentation)
FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollection, FW_CPresentation)
FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollectionIterator, FW_CPrivDisplayFrame)
FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollection, FW_CPrivDisplayFrame)

#ifdef FW_USE_TEMPLATE_PRAGMAS

#pragma template_access public
#pragma template FW_TOrderedCollection<FW_CPresentation>
#pragma template FW_TOrderedCollectionIterator<FW_CPresentation>
#pragma template FW_TOrderedCollection<FW_CPrivDisplayFrame>
#pragma template FW_TOrderedCollectionIterator<FW_CPrivDisplayFrame>

#endif

//========================================================================================
//	class FW_CPrivDisplayFrame
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CPrivDisplayFrame::FW_CPrivDisplayFrame
//----------------------------------------------------------------------------------------

FW_CPrivDisplayFrame::FW_CPrivDisplayFrame(Environment *ev, FW_CFrame* frame) :
	fFrameID(frame->GetID(ev)),
	fODFrame(frame->GetODFrame(ev)),
	fFrame(frame)
{
	fODFrame->Acquire(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CPrivDisplayFrame::FW_CPrivDisplayFrame
//----------------------------------------------------------------------------------------

FW_CPrivDisplayFrame::FW_CPrivDisplayFrame(Environment *ev, ODID frameID) :
	fFrameID(frameID),
	fODFrame(NULL),
	fFrame(NULL)
{
FW_UNUSED(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CPrivDisplayFrame::~FW_CPrivDisplayFrame
//----------------------------------------------------------------------------------------

FW_CPrivDisplayFrame::~FW_CPrivDisplayFrame()
{
	Release(somGetGlobalEnvironment());
}

//----------------------------------------------------------------------------------------
//	FW_CPrivDisplayFrame::Release
//----------------------------------------------------------------------------------------

void FW_CPrivDisplayFrame::Release(Environment *ev)
{
	if (fODFrame)
	{
		fODFrame->SetPartInfo(ev, NULL);
		fODFrame->Release(ev);
		fODFrame = NULL;
	}
}

//========================================================================================
//	class FW_CPresentation
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CPresentation::FW_CPresentation
//----------------------------------------------------------------------------------------

FW_CPresentation::FW_CPresentation(Environment *ev, 
								FW_CPart* thePart, 
								FW_CSelection* selection,
								ODTypeToken presentationType,
								ODTypeToken defaultEmbeddedFrameViewType) :
	fPart(thePart),
	fFrames(NULL),
	fSelection(selection),
	fPresentationType(presentationType),
	fFocusSet(NULL),
	fArbitrator(NULL),
	fPartWindowFrame(NULL),
	fViewAsThumbnail(NULL),
	fViewAsSmallIcon(NULL),
	fViewAsLargeIcon(NULL),
	fDefaultEmbeddedFrameViewType(defaultEmbeddedFrameViewType),
	fDragPending(FALSE)
{
	FW_ASSERT(thePart);
	
	fFrames = FW_NEW(FW_TOrderedCollection<FW_CPrivDisplayFrame>, ());

	fArbitrator = fPart->GetSession(ev)->GetArbitrator(ev);

	// ----- By default add all common foci to the presentation
	// 		(floating windows & dialogs will updated later)
	fFocusSet = fArbitrator->CreateFocusSet(ev);
	fFocusSet->Add(ev, FW_CPart::gKeyFocusToken);
	fFocusSet->Add(ev, FW_CPart::gMenuFocusToken);
	fFocusSet->Add(ev, FW_CPart::gSelectionFocusToken);
	fFocusSet->Add(ev, FW_CPart::gClipboardFocusToken);
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::~FW_CPresentation
//----------------------------------------------------------------------------------------

FW_CPresentation::~FW_CPresentation()
{
	FW_ASSERT(fViewAsThumbnail == NULL);		// Should already have been released
	FW_ASSERT(fViewAsSmallIcon == NULL);
	FW_ASSERT(fViewAsLargeIcon == NULL);

	delete fSelection;
	
	Environment* ev = somGetGlobalEnvironment();
	
	if (fFrames != NULL)
	{
		FW_CPrivDisplayFrame* displayFrame;
		while ((displayFrame = fFrames->First()) != NULL)
			PrivDeleteDisplayFrame(ev, displayFrame);
		
		delete fFrames;
	}

	delete fFocusSet;
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::Invalidate
//----------------------------------------------------------------------------------------
// invalidShape should be in content coordinates

void FW_CPresentation::Invalidate(Environment *ev, ODShape* invalidShape, ODID sequenceNumber)
{
	FW_CPresentationFrameIterator ite(ev, this);
	for (FW_CFrame* frame = ite.First(ev); ite.IsNotComplete(ev); frame = ite.Next(ev))
	{
		if (sequenceNumber == 0 || sequenceNumber == frame->GetSequenceNumber(ev))
		{
			FW_CAcquiredODShape aqInvalidShape;
			if (invalidShape != NULL)
			{
				FW_CAcquiredODTransform aqInternalTransform = frame->AcquireInternalTransform(ev, NULL);
				aqInvalidShape = invalidShape->Copy(ev);
				aqInvalidShape->Transform(ev, aqInternalTransform);
			}
			
			frame->GetODFrame(ev)->Invalidate(ev, aqInvalidShape, NULL);		// if aqInvalidShape is NULL OpenDoc will use the aggregate
																				// clip shape
		}
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::Validate
//----------------------------------------------------------------------------------------
// validShape should be in content coordinates

void FW_CPresentation::Validate(Environment *ev, ODShape* validShape, ODID sequenceNumber)
{
	FW_CPresentationFrameIterator ite(ev, this);
	for (FW_CFrame* frame = ite.First(ev); ite.IsNotComplete(ev); frame = ite.Next(ev))
	{
		if (sequenceNumber == 0 || sequenceNumber == frame->GetSequenceNumber(ev))
		{
			FW_CAcquiredODShape aqValidShape;
			if (validShape != NULL)
			{
				FW_CAcquiredODTransform aqInternalTransform = frame->AcquireInternalTransform(ev, NULL);
				aqValidShape = validShape->Copy(ev);
				aqValidShape->Transform(ev, aqInternalTransform);
			}
			
			frame->GetODFrame(ev)->Validate(ev, aqValidShape, NULL);		// if aqInvalidShape is NULL OpenDoc will use the aggregate
																			// clip shape
		}
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::Invalidate
//----------------------------------------------------------------------------------------
// invalidRect should be in content coordinates

void FW_CPresentation::Invalidate(Environment* ev, const FW_CRect& invalidRect, ODID sequenceNumber)
{
	FW_CAcquiredODShape aqShape = ::FW_NewODShape(ev, invalidRect);
	Invalidate(ev, aqShape, sequenceNumber);
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::Validate
//----------------------------------------------------------------------------------------
// validRect should be in content coordinates

void FW_CPresentation::Validate(Environment* ev, const FW_CRect& validRect, ODID sequenceNumber)
{
	FW_CAcquiredODShape aqShape = ::FW_NewODShape(ev, validRect);
	Validate(ev, aqShape, sequenceNumber);
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::ContentUpdated
//------------------------------------------------------------------------

void FW_CPresentation::ContentUpdated(Environment* ev, ODUpdateID updateID)
{
	// When ContentUpdated is called because of updating a link, a change ID is supplied.
	// Otherwise, we need to create a unique change ID -
	if (updateID == kODUnknownUpdate)
		updateID = fPart->GetSession(ev)->UniqueUpdateID(ev);

	FW_CPresentationFrameIterator ite(ev, this);
	for (FW_CFrame* frame = ite.First(ev); ite.IsNotComplete(ev); frame = ite.Next(ev))
	{
		frame->GetODFrame(ev)->ContentUpdated(ev, updateID);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::PrivAddFrame
//----------------------------------------------------------------------------------------

void FW_CPresentation::PrivAddFrame(Environment *ev, FW_CFrame* frame)
{
	// ----- set up the last active frame ------
	if (fPart->GetLastActiveFrame(ev) == NULL)
	{
		ODFocusSet* focusSet = frame->GetFocusSet(ev);
		if (focusSet->Contains(ev, FW_CPart::gSelectionFocusToken))
			fPart->PrivSetLastActiveFrame(frame);
	}
	
	fFrames->AddLast(new FW_CPrivDisplayFrame(ev, frame));
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::PrivGetDisplayFrame
//----------------------------------------------------------------------------------------

FW_CPrivDisplayFrame* FW_CPresentation::PrivGetDisplayFrame(FW_CFrame* frame) const
{
	FW_TOrderedCollectionIterator<FW_CPrivDisplayFrame> ite(fFrames);
	for (FW_CPrivDisplayFrame* displayFrame = ite.First(); ite.IsNotComplete(); displayFrame = ite.Next())
	{
		if (displayFrame->fFrame == frame)
			return displayFrame;
	}
	
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::PrivDeleteDisplayFrame
//----------------------------------------------------------------------------------------

void FW_CPresentation::PrivDeleteDisplayFrame(Environment *ev, FW_CPrivDisplayFrame* displayFrame)
{
	// ----- First Remove it from the list -----
	fFrames->Remove(displayFrame);

	// ----- if this frame was an "Open in Window" frame then ....
	if (fPartWindowFrame == displayFrame->fFrame)
	{
		fPartWindowFrame = NULL;	
		PrivInvalidateIconedFrame(ev);
	}
	
	// ----- Now I can delete it
	delete displayFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::PrivInvalidateIconedFrame
//----------------------------------------------------------------------------------------

void FW_CPresentation::PrivInvalidateIconedFrame(Environment *ev)
{
	FW_CPresentationFrameIterator ite(ev, this);
	for (FW_CFrame* aFrame = ite.First(ev); ite.IsNotComplete(ev); aFrame =  ite.Next(ev))
	{
		ODTypeToken viewType = aFrame->GetViewType(ev);
		if (viewType == FW_CPart::gViewAsLargeIconToken || viewType == FW_CPart::gViewAsSmallIconToken)
			aFrame->GetODFrame(ev)->Invalidate(ev, NULL, NULL);
	}		
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::ReleaseAll
//----------------------------------------------------------------------------------------

void FW_CPresentation::ReleaseAll(Environment *ev)
{
	FW_TOrderedCollectionIterator<FW_CPrivDisplayFrame> ite(fFrames);
	for (FW_CPrivDisplayFrame* displayFrame = ite.First(); ite.IsNotComplete(); displayFrame = ite.Next())
	{
		displayFrame->Release(ev);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::PrivFrameRemoved
//----------------------------------------------------------------------------------------

void FW_CPresentation::PrivFrameRemoved(Environment *ev, FW_CFrame* frame, FW_Boolean toStorage)
{
	FW_Boolean wasLastActive = (fPart->GetLastActiveFrame(ev) == frame);
	
	// ----- Notify the frame -----
	frame->FrameRemoved(ev, toStorage);
	
	// ----- Remove it from the list -----
	FW_CPrivDisplayFrame* displayFrame = PrivGetDisplayFrame(frame);
	FW_ASSERT(displayFrame);

	PrivDeleteDisplayFrame(ev, displayFrame);

	// ----- delete it if not shared -----
	if (!frame->PrivInSharedWindow(ev))
		delete frame;
	
	// ----- Set the last active frame to the first frame of this presentation -----	
	if (wasLastActive)
	{
		FW_CPrivDisplayFrame* displayFrame = fFrames->First();
		fPart->PrivSetLastActiveFrame(displayFrame ? displayFrame->fFrame : NULL);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::CountFrame
//----------------------------------------------------------------------------------------

unsigned long FW_CPresentation::CountFrame(Environment *ev) const
{
FW_UNUSED(ev);
	return fFrames->Count();
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::Embed
//----------------------------------------------------------------------------------------

void FW_CPresentation::Embed(Environment*	ev, 
								ODPart*		embeddedPart,
								ODFrame* 	embeddedFrame,
								ODType 		frameType,
								FW_MProxy*	proxy,
								ODShape*	frameShape,
								ODTypeToken viewType,
								ODTypeToken	presentationType,
								ODID		frameGroupID,
								FW_Boolean	isOverlaid,
								FW_Boolean	subFrame)
{
FW_UNUSED(ev);
FW_UNUSED(embeddedPart);
FW_UNUSED(embeddedFrame);
FW_UNUSED(frameType);
FW_UNUSED(proxy);
FW_UNUSED(frameShape);
FW_UNUSED(viewType);
FW_UNUSED(presentationType);
FW_UNUSED(frameGroupID);
FW_UNUSED(isOverlaid);
FW_UNUSED(subFrame);
	FW_DEBUG_MESSAGE("FW_CPresentation::Embed  Should never have been called");
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::RemoveSiblingFrame
//----------------------------------------------------------------------------------------

void FW_CPresentation::RemoveSiblingFrame(Environment* ev, FW_CFrame* frameToRemove)
{
	FW_ASSERT(frameToRemove->IsRequestedFrame(ev));	// Cannot remove a frame not requested
	
	FW_CAcquiredODFrame embeddingFrame = frameToRemove->AcquireContainingFrame(ev);
	FW_CAcquiredODPart embeddingPart = embeddingFrame->AcquirePart(ev);
	
	embeddingPart->RemoveEmbeddedFrame(ev, frameToRemove->GetODFrame(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::RequestSiblingFrame
//----------------------------------------------------------------------------------------
//	returns kODNULLID if the request was not fulfilled

ODID FW_CPresentation::RequestSiblingFrame(Environment* ev,
											FW_CFrame* baseFrame, 
											ODShape* suggestedFrameShape,
											ODTypeToken viewType,
											FW_Boolean isOverlaid)
{
	FW_CAcquiredODFrame embeddingFrame = baseFrame->AcquireContainingFrame(ev);
	FW_CAcquiredODPart embeddingPart = embeddingFrame->AcquirePart(ev);
	
	FW_CAcquiredODShape frameShape = suggestedFrameShape->Copy(ev);
	FW_CAcquiredODFrame newFrame = embeddingPart->RequestEmbeddedFrame(ev,
																		embeddingFrame,
																		baseFrame->GetODFrame(ev),
																		frameShape,
																		fPart->GetODPart(ev),
																		viewType,
																		fPresentationType,
																		isOverlaid);
	ODID newSequence = kODNULLID;
	
	if (newFrame != NULL)
	{
		FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, newFrame);
		
		ODID newFrameGroup = fwFrame->GetFrameGroup(ev);
		newSequence = fwFrame->GetSequenceNumber(ev);
	
		// ----- Set the RequestedFrame flag -----
		FW_CPresentationFrameIterator ite(ev, this, newFrameGroup, newSequence);
		for (FW_CFrame* aFrame = ite.First(ev); ite.IsNotComplete(ev); aFrame =  ite.Next(ev))
		{
			aFrame->PrivSetRequestedFrame(ev);
		}	
	}
	
	return newSequence;
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::OpenPartWindow
//----------------------------------------------------------------------------------------

ODID FW_CPresentation::OpenPartWindow(Environment* ev, FW_CFrame* sourceFrame, ODFacet* sourceFacet)
{	
FW_ASSERT(sourceFrame);

	FW_CWindow* window = NULL;
	
	if (fPartWindowFrame != NULL)
	{
		window = fPartWindowFrame->GetWindow(ev);
	}
	else
	{
		if (sourceFacet == NULL)
		{
			FW_CFrameFacetIterator ite(ev, sourceFrame);
			sourceFacet = ite.First(ev);
		}
		FW_ASSERT(sourceFacet != NULL);
		
		// ----- Invalid all my frames if iconized -----
		PrivInvalidateIconedFrame(ev);
		
		window = sourceFrame->NewPartWindow(ev, sourceFacet);						
		window->Show(ev);	// ATTENTION: Show should be before Select
		
		FW_CAcquiredODWindow odWindow = window->AcquireODWindow(ev);
		
		fPartWindowFrame = FW_CFrame::ODtoFWFrame(ev, odWindow->GetRootFacet(ev)->GetFrame(ev));
		FW_ASSERT(fPartWindowFrame != NULL);
	}
		
	window->Select(ev);
	
	return window->GetID(ev);
}

//----------------------------------------------------------------------------------------
// FW_CPresentation::PrivAcquireViewAs
//----------------------------------------------------------------------------------------

FW_CViewAs* FW_CPresentation::PrivAcquireViewAs(Environment* ev, FW_CFrame* frame, ODTypeToken viewAsToken)
{
	FW_ASSERT(viewAsToken == FW_CPart::gViewAsThumbnailToken ||
			viewAsToken == FW_CPart::gViewAsSmallIconToken ||
			viewAsToken == FW_CPart::gViewAsLargeIconToken);
			
	FW_CViewAs* viewAs = NULL;
	if (viewAsToken == FW_CPart::gViewAsThumbnailToken)
	{
		if (fViewAsThumbnail != NULL)
			fViewAsThumbnail->Acquire();
		else
			fViewAsThumbnail = new FW_CViewAsThumbnail(ev);
		viewAs = fViewAsThumbnail;
	}
	else if (viewAsToken == FW_CPart::gViewAsSmallIconToken)
	{
		if (fViewAsSmallIcon != NULL)
			fViewAsSmallIcon->Acquire();
		else
			fViewAsSmallIcon = new FW_CViewAsSmallIcon(ev);
		viewAs = fViewAsSmallIcon;			
	}
	else
	{
		if (fViewAsLargeIcon != NULL)
			fViewAsLargeIcon->Acquire();
		else
			fViewAsLargeIcon = new FW_CViewAsLargeIcon(ev);
		viewAs = fViewAsLargeIcon;			
	}
			
	viewAs->Load(ev, frame);
	return viewAs;
}

//----------------------------------------------------------------------------------------
// FW_CPresentation::PrivReleaseViewAs
//----------------------------------------------------------------------------------------

void FW_CPresentation::PrivReleaseViewAs(FW_CViewAs* viewAs)
{
	FW_ASSERT(viewAs == fViewAsThumbnail || viewAs == fViewAsSmallIcon || viewAs == fViewAsLargeIcon);
	
	if (viewAs->Release() == 0)
	{
		if (viewAs == fViewAsThumbnail)
			fViewAsThumbnail = NULL;
		else if (viewAs == fViewAsSmallIcon)
			fViewAsSmallIcon = NULL;
		else
			fViewAsLargeIcon = NULL;
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::UpdateViewAs
//----------------------------------------------------------------------------------------

void FW_CPresentation::UpdateViewAs(Environment* ev)
{
	if (fViewAsThumbnail != NULL)
		fViewAsThumbnail->Unload(ev);
	
	if (fViewAsSmallIcon != NULL)
		fViewAsSmallIcon->Unload(ev);
	
	if (fViewAsLargeIcon != NULL)
		fViewAsSmallIcon->Unload(ev);
}

//----------------------------------------------------------------------------------------
// FW_CPresentation::GetPartWindow
//----------------------------------------------------------------------------------------

FW_CWindow* FW_CPresentation::GetPartWindow(Environment* ev) const
{
	return fPartWindowFrame ? fPartWindowFrame->GetWindow(ev) : NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::AdoptFocusSet
//----------------------------------------------------------------------------------------

void FW_CPresentation::AdoptFocusSet(Environment* ev, ODFocusSet* newFocusSet)
{
#ifdef FW_DEBUG
	// No frame should be active when changing the focus set because it is used
	// during activation/deactivation
	FW_CPresentationFrameIterator ite(ev, this);
	for (FW_CFrame* aFrame = ite.First(ev); ite.IsNotComplete(ev); aFrame =  ite.Next(ev))
	{
		FW_ASSERT(aFrame->HasSelectionFocus(ev) == FALSE);
	}
#endif
	
	delete fFocusSet;
	if (newFocusSet)
		fFocusSet = newFocusSet;
	else
		fFocusSet = fArbitrator->CreateFocusSet(ev);	// Empty focus set
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::SetFocusSet
//----------------------------------------------------------------------------------------

void FW_CPresentation::SetFocusSet(Environment* ev, ODFocusSet* newFocusSet)
{
#ifdef FW_DEBUG
	// No frame should be active when changing the focus set because it is used
	// during activation/deactivation
	FW_CPresentationFrameIterator ite(ev, this);
	for (FW_CFrame* aFrame = ite.First(ev); ite.IsNotComplete(ev); aFrame =  ite.Next(ev))
	{
		FW_ASSERT(aFrame->HasSelectionFocus(ev) == FALSE);
	}
#else
FW_UNUSED(ev);
#endif
		
	fFocusSet = newFocusSet;
}

//----------------------------------------------------------------------------------------
//	FW_CPresentation::PrivNewFrame
//----------------------------------------------------------------------------------------
//	PrivNewFrame is used internaly for example by the about dialog. Part should never
//	have to subclass FW_CPresentation. Use FW_CPart::NewFrame instead.

FW_CFrame* FW_CPresentation::PrivNewFrame(Environment *ev,
										ODFrame* odFrame, 
										FW_Boolean fromStorage)
{
FW_UNUSED(ev);
FW_UNUSED(odFrame);
FW_UNUSED(fromStorage);
	return NULL;
}