//========================================================================================
//
//	File:				FWPrmise.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWPRMISE_H
#include "FWPrmise.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWCLNINF_H
#include "FWClnInf.h"
#endif

#ifndef FWINTER_H
#include "FWInter.h"
#endif

#ifndef FWLNKSRC_H
#include "FWLnkSrc.h"
#endif

// ----- OS Layer Includes -----

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

// ----- Foundation Layer Includes -----

#ifndef FWMEMORY_H
#include "FWMemory.h"
#endif

#ifndef SLPRISTR_H
#include "SLPriStr.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODClipboard_xh
#include <Clipbd.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODStorageUnitView_xh
#include <SUView.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#if defined(__MWERKS__) && GENERATING68K
// A hack to work around a bug
#pragma import list somGetGlobalEnvironment
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment fwpart2
#endif

//========================================================================================
//	Template Instantiations
//========================================================================================

FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollectionIterator, FW_CPromise)
FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollection, FW_CPromise)

#ifdef FW_USE_TEMPLATE_PRAGMAS

#pragma template_access public
#pragma template FW_TOrderedCollection<FW_CPromise>
#pragma template FW_TOrderedCollectionIterator<FW_CPromise>

#endif

//========================================================================================
//	class FW_CPromise
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CPromise::FW_CPromise
//----------------------------------------------------------------------------------------

FW_CPromise::FW_CPromise(Environment*ev, FW_EStorageKinds storageKind, FW_CCloneInfo* cloneInfo) :
	fScopeFrame(cloneInfo->GetScopeFrame(ev)),
	fDataInterchange(cloneInfo->GetDataInterchange(ev)),
	fStorageKind(storageKind),
	fUpdateID(0),
	fPromiseCount(0)
{
	FW_ASSERT(fStorageKind != FW_kLinkStorage);
	FW_ASSERT(fStorageKind != FW_kPartStorage);	// can promise on the part storage
	FW_ASSERT(fStorageKind != FW_kFileStorage);	// can promise on a file storage

	fUpdateID = fScopeFrame->GetPart(ev)->GetSession(ev)->GetClipboard(ev)->GetUpdateID(ev);	
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::FW_CPromise
//----------------------------------------------------------------------------------------

FW_CPromise::FW_CPromise(Environment*ev, FW_CLinkSource* linkSource, FW_CCloneInfo* cloneInfo) :
	fScopeFrame(cloneInfo->GetScopeFrame(ev)),
	fDataInterchange(cloneInfo->GetDataInterchange(ev)),
	fStorageKind(FW_kLinkStorage),
	fUpdateID(0),
	fLinkSource(linkSource),
	fPromiseCount(0)
{
	FW_ASSERT(fLinkSource != NULL);
	fLinkSource->PrivSetPromise(ev, this);
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::~FW_CPromise
//----------------------------------------------------------------------------------------

FW_CPromise::~FW_CPromise()
{
	Environment* ev = somGetGlobalEnvironment();
	if (fStorageKind == FW_kLinkStorage)
	{
		FW_ASSERT(fLinkSource != NULL);
		fLinkSource->PrivSetPromise(ev, NULL);
	}
	
	fDataInterchange->PrivPromiseDeleted(ev, this);
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::Promise
//----------------------------------------------------------------------------------------

void FW_CPromise::Promise(Environment* ev, ODStorageUnit* storageUnit, ODPropertyName propertyName, ODValueType valueType)
{	
	fDataInterchange->PrivPromise(ev, this, storageUnit, propertyName, valueType);
	fPromiseCount++;
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::PrivHandleFulfillPromise
//----------------------------------------------------------------------------------------

long FW_CPromise::PrivHandleFulfillPromise(Environment* ev, ODStorageUnitView* promiseSUView)
{
	ODDraft* fromDraft = fScopeFrame->GetPart(ev)->GetDraft(ev);
	ODDraft* dstDraft = promiseSUView->GetStorageUnit(ev)->GetDraft(ev);

	// Get the cloneKind from the kODPropCloneKindUsed property, if it's available
	ODCloneKind cloneKind;
	if (FW_SUExistsThenFocus(ev, promiseSUView->GetStorageUnit(ev), kODPropCloneKindUsed, kODCloneKind) )
	{
		FW_CByteArray byteArray;
		promiseSUView->GetStorageUnit(ev)->GetValue(ev, sizeof(ODCloneKind), byteArray);
		byteArray.CopyBuffer(&cloneKind, sizeof(ODCloneKind));
	}
	else
		cloneKind = (fStorageKind == FW_kLinkStorage) ? kODCloneToLink : kODCloneCopy;

	FW_CAcquireODPropertyName propertyName(ev, promiseSUView);
	FW_CAcquireODType valueType = promiseSUView->GetType(ev);	

	FW_CCloneInfo cloneInfo(ev, fromDraft, fScopeFrame, cloneKind);	
	cloneInfo.BeginClone(ev, dstDraft);

	FulfillPromise(ev, promiseSUView, propertyName, valueType, &cloneInfo);

	cloneInfo.EndClone(ev);
	
	return --fPromiseCount;
}
